#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright: (c) 2018, Ansible Project
# Copyright: (c) 2018, Abhijeet Kasurde <akasurde@redhat.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = r'''
---
module: digital_ocean_load_balancer_info
short_description: Gather information about DigitalOcean load balancers
description:
    - This module can be used to gather information about DigitalOcean provided load balancers.
    - This module was called C(digital_ocean_load_balancer_facts) before Ansible 2.9. The usage did not change.
author: "Abhijeet Kasurde (@Akasurde)"
options:
  load_balancer_id:
    description:
     - Load balancer ID that can be used to identify and reference a load_balancer.
    required: false
    type: str
requirements:
  - "python >= 2.6"
extends_documentation_fragment:
- community.digitalocean.digital_ocean.documentation

'''


EXAMPLES = r'''
- name: Gather information about all load balancers
  community.digitalocean.digital_ocean_load_balancer_info:
    oauth_token: "{{ oauth_token }}"

- name: Gather information about load balancer with given id
  community.digitalocean.digital_ocean_load_balancer_info:
    oauth_token: "{{ oauth_token }}"
    load_balancer_id: "4de7ac8b-495b-4884-9a69-1050c6793cd6"

- name: Get name from load balancer id
  community.digitalocean.digital_ocean_load_balancer_info:
  register: resp_out
- set_fact:
    load_balancer_name: "{{ item.name }}"
  loop: "{{ resp_out.data | community.general.json_query(name) }}"
  vars:
    name: "[?id=='4de7ac8b-495b-4884-9a69-1050c6793cd6']"
- debug:
    var: load_balancer_name
'''


RETURN = r'''
data:
    description: DigitalOcean Load balancer information
    returned: success
    type: list
    sample: [
        {
          "id": "4de7ac8b-495b-4884-9a69-1050c6793cd6",
          "name": "example-lb-01",
          "ip": "104.131.186.241",
          "algorithm": "round_robin",
          "status": "new",
          "created_at": "2017-02-01T22:22:58Z",
          ...
        },
    ]
'''

from traceback import format_exc
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.digitalocean.plugins.module_utils.digital_ocean import DigitalOceanHelper
from ansible.module_utils._text import to_native


def core(module):
    load_balancer_id = module.params.get('load_balancer_id', None)
    rest = DigitalOceanHelper(module)

    base_url = 'load_balancers?'
    if load_balancer_id is not None:
        response = rest.get("%s/%s" % (base_url, load_balancer_id))
        status_code = response.status_code

        if status_code != 200:
            module.fail_json(msg="Failed to retrieve load balancers for DigitalOcean")

        resp_json = response.json
        load_balancer = resp_json['load_balancer']
    else:
        load_balancer = rest.get_paginated_data(base_url=base_url, data_key_name='load_balancers')

    module.exit_json(changed=False, data=load_balancer)


def main():
    argument_spec = DigitalOceanHelper.digital_ocean_argument_spec()
    argument_spec.update(
        load_balancer_id=dict(type='str', required=False),
    )
    module = AnsibleModule(argument_spec=argument_spec)
    if module._name in ('digital_ocean_load_balancer_facts', 'community.digitalocean.digital_ocean_load_balancer_facts'):
        module.deprecate("The 'digital_ocean_load_balancer_facts' module has been renamed to 'digital_ocean_load_balancer_info'",
                         version='2.0.0', collection_name='community.digitalocean')  # was Ansible 2.13

    try:
        core(module)
    except Exception as e:
        module.fail_json(msg=to_native(e), exception=format_exc())


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# @author: Gaurav Rastogi (grastogi@avinetworks.com)
#          Eric Anderson (eanderson@avinetworks.com)
# module_check: supported
# Avi Version: 17.1.1
#
# Copyright: (c) 2017 Gaurav Rastogi, <grastogi@avinetworks.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = '''
---
module: avi_systemconfiguration
author: Gaurav Rastogi (@grastogi23) <grastogi@avinetworks.com>

short_description: Module for setup of SystemConfiguration Avi RESTful Object
description:
    - This module is used to configure SystemConfiguration object
    - more examples at U(https://github.com/avinetworks/devops)
requirements: [ avisdk ]
options:
    state:
        description:
            - The state that should be applied on the entity.
        default: present
        choices: ["absent", "present"]
    avi_api_update_method:
        description:
            - Default method for object update is HTTP PUT.
            - Setting to patch will override that behavior to use HTTP PATCH.
        default: put
        choices: ["put", "patch"]
    avi_api_patch_op:
        description:
            - Patch operation to use when using avi_api_update_method as patch.
        choices: ["add", "replace", "delete"]
    admin_auth_configuration:
        description:
            - Adminauthconfiguration settings for systemconfiguration.
    default_license_tier:
        description:
            - Specifies the default license tier which would be used by new clouds.
            - Enum options - ENTERPRISE_16, ENTERPRISE_18.
            - Field introduced in 17.2.5.
            - Default value when not specified in API or module is interpreted by Avi Controller as ENTERPRISE_18.
    dns_configuration:
        description:
            - Dnsconfiguration settings for systemconfiguration.
    dns_virtualservice_refs:
        description:
            - Dns virtualservices hosting fqdn records for applications across avi vantage.
            - If no virtualservices are provided, avi vantage will provide dns services for configured applications.
            - Switching back to avi vantage from dns virtualservices is not allowed.
            - It is a reference to an object of type virtualservice.
    docker_mode:
        description:
            - Boolean flag to set docker_mode.
            - Default value when not specified in API or module is interpreted by Avi Controller as False.
        type: bool
    email_configuration:
        description:
            - Emailconfiguration settings for systemconfiguration.
    global_tenant_config:
        description:
            - Tenantconfiguration settings for systemconfiguration.
    linux_configuration:
        description:
            - Linuxconfiguration settings for systemconfiguration.
    mgmt_ip_access_control:
        description:
            - Configure ip access control for controller to restrict open access.
    ntp_configuration:
        description:
            - Ntpconfiguration settings for systemconfiguration.
    portal_configuration:
        description:
            - Portalconfiguration settings for systemconfiguration.
    proxy_configuration:
        description:
            - Proxyconfiguration settings for systemconfiguration.
    secure_channel_configuration:
        description:
            - Configure secure channel properties.
            - Field introduced in 18.1.4, 18.2.1.
    snmp_configuration:
        description:
            - Snmpconfiguration settings for systemconfiguration.
    ssh_ciphers:
        description:
            - Allowed ciphers list for ssh to the management interface on the controller and service engines.
            - If this is not specified, all the default ciphers are allowed.
    ssh_hmacs:
        description:
            - Allowed hmac list for ssh to the management interface on the controller and service engines.
            - If this is not specified, all the default hmacs are allowed.
    url:
        description:
            - Avi controller URL of the object.
    uuid:
        description:
            - Unique object identifier of the object.
    welcome_workflow_complete:
        description:
            - This flag is set once the initial controller setup workflow is complete.
            - Field introduced in 18.2.3.
            - Default value when not specified in API or module is interpreted by Avi Controller as False.
        type: bool
extends_documentation_fragment:
- community.network.avi

'''

EXAMPLES = """
- name: Example to create SystemConfiguration object
  community.network.avi_systemconfiguration:
    controller: 10.10.25.42
    username: admin
    password: something
    state: present
    name: sample_systemconfiguration
"""

RETURN = '''
obj:
    description: SystemConfiguration (api/systemconfiguration) object
    returned: success, changed
    type: dict
'''

from ansible.module_utils.basic import AnsibleModule
try:
    from ansible_collections.community.network.plugins.module_utils.network.avi.avi import (
        avi_common_argument_spec, avi_ansible_api, HAS_AVI)
except ImportError:
    HAS_AVI = False


def main():
    argument_specs = dict(
        state=dict(default='present',
                   choices=['absent', 'present']),
        avi_api_update_method=dict(default='put',
                                   choices=['put', 'patch']),
        avi_api_patch_op=dict(choices=['add', 'replace', 'delete']),
        admin_auth_configuration=dict(type='dict',),
        default_license_tier=dict(type='str',),
        dns_configuration=dict(type='dict',),
        dns_virtualservice_refs=dict(type='list',),
        docker_mode=dict(type='bool',),
        email_configuration=dict(type='dict',),
        global_tenant_config=dict(type='dict',),
        linux_configuration=dict(type='dict',),
        mgmt_ip_access_control=dict(type='dict',),
        ntp_configuration=dict(type='dict',),
        portal_configuration=dict(type='dict',),
        proxy_configuration=dict(type='dict',),
        secure_channel_configuration=dict(type='dict',),
        snmp_configuration=dict(type='dict',),
        ssh_ciphers=dict(type='list',),
        ssh_hmacs=dict(type='list',),
        url=dict(type='str',),
        uuid=dict(type='str',),
        welcome_workflow_complete=dict(type='bool',),
    )
    argument_specs.update(avi_common_argument_spec())
    module = AnsibleModule(
        argument_spec=argument_specs, supports_check_mode=True)
    if not HAS_AVI:
        return module.fail_json(msg=(
            'Avi python API SDK (avisdk>=17.1) or requests is not installed. '
            'For more details visit https://github.com/avinetworks/sdk.'))
    return avi_ansible_api(module, 'systemconfiguration',
                           set([]))


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# Copyright 2016 Doalitic.
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

"""
Brook.io external inventory script
==================================

Generates inventory that Ansible can understand by making API requests to Brook.io via the libbrook
library. Hence, such dependency must be installed in the system to run this script.

The default configuration file is named 'brook.ini' and is located alongside this script. You can
choose any other file by setting the BROOK_INI_PATH environment variable.

If param 'project_id' is left blank in 'brook.ini', the inventory includes all the instances in
projects where the requesting user belongs. Otherwise, only instances from the given project are
included, provided the requesting user belongs to it.

The following variables are established for every host. They can be retrieved from the hostvars
dictionary.
 - brook_pid: str
 - brook_name: str
 - brook_description: str
 - brook_project: str
 - brook_template: str
 - brook_region: str
 - brook_zone: str
 - brook_status: str
 - brook_tags: list(str)
 - brook_internal_ips: list(str)
 - brook_external_ips: list(str)
 - brook_created_at
 - brook_updated_at
 - ansible_ssh_host

Instances are grouped by the following categories:
 - tag:
   A group is created for each tag. E.g. groups 'tag_foo' and 'tag_bar' are created if there exist
   instances with tags 'foo' and/or 'bar'.
 - project:
   A group is created for each project. E.g. group 'project_test' is created if a project named
   'test' exist.
 - status:
   A group is created for each instance state. E.g. groups 'status_RUNNING' and 'status_PENDING'
   are created if there are instances in running and pending state.

Examples:
  Execute uname on all instances in project 'test'
  $ ansible -i brook.py project_test -m shell -a "/bin/uname -a"

  Install nginx on all debian web servers tagged with 'www'
  $ ansible -i brook.py tag_www -m apt -a "name=nginx state=present"

  Run site.yml playbook on web servers
  $ ansible-playbook -i brook.py site.yml -l tag_www

Support:
  This script is tested on Python 2.7 and 3.4. It may work on other versions though.

Author: Francisco Ros <fjros@doalitic.com>
Version: 0.2
"""


import sys
import os

from ansible.module_utils.six.moves.configparser import SafeConfigParser as ConfigParser

import json

try:
    import libbrook
except Exception:
    sys.exit('Brook.io inventory script requires libbrook. See https://github.com/doalitic/libbrook')


class BrookInventory:

    _API_ENDPOINT = 'https://api.brook.io'

    def __init__(self):
        self._configure_from_file()
        self.client = self.get_api_client()
        self.inventory = self.get_inventory()

    def _configure_from_file(self):
        """Initialize from .ini file.

        Configuration file is assumed to be named 'brook.ini' and to be located on the same
        directory than this file, unless the environment variable BROOK_INI_PATH says otherwise.
        """

        brook_ini_default_path = \
            os.path.join(os.path.dirname(os.path.realpath(__file__)), 'brook.ini')
        brook_ini_path = os.environ.get('BROOK_INI_PATH', brook_ini_default_path)

        config = ConfigParser(defaults={
            'api_token': '',
            'project_id': ''
        })
        config.read(brook_ini_path)
        self.api_token = config.get('brook', 'api_token')
        self.project_id = config.get('brook', 'project_id')

        if not self.api_token:
            sys.exit('You must provide (at least) your Brook.io API token to generate the dynamic '
                     'inventory.')

    def get_api_client(self):
        """Authenticate user via the provided credentials and return the corresponding API client.
        """

        # Get JWT token from API token
        #
        unauthenticated_client = libbrook.ApiClient(host=self._API_ENDPOINT)
        auth_api = libbrook.AuthApi(unauthenticated_client)
        api_token = libbrook.AuthTokenRequest()
        api_token.token = self.api_token
        jwt = auth_api.auth_token(token=api_token)

        # Create authenticated API client
        #
        return libbrook.ApiClient(host=self._API_ENDPOINT,
                                  header_name='Authorization',
                                  header_value='Bearer %s' % jwt.token)

    def get_inventory(self):
        """Generate Ansible inventory.
        """

        groups = dict()
        meta = dict()
        meta['hostvars'] = dict()

        instances_api = libbrook.InstancesApi(self.client)
        projects_api = libbrook.ProjectsApi(self.client)
        templates_api = libbrook.TemplatesApi(self.client)

        # If no project is given, get all projects the requesting user has access to
        #
        if not self.project_id:
            projects = [project.id for project in projects_api.index_projects()]
        else:
            projects = [self.project_id]

        # Build inventory from instances in all projects
        #
        for project_id in projects:
            project = projects_api.show_project(project_id=project_id)
            for instance in instances_api.index_instances(project_id=project_id):
                # Get template used for this instance if known
                template = templates_api.show_template(template_id=instance.template) if instance.template else None

                # Update hostvars
                try:
                    meta['hostvars'][instance.name] = \
                        self.hostvars(project, instance, template, instances_api)
                except libbrook.rest.ApiException:
                    continue

                # Group by project
                project_group = 'project_%s' % project.name
                if project_group in groups:
                    groups[project_group].append(instance.name)
                else:
                    groups[project_group] = [instance.name]

                # Group by status
                status_group = 'status_%s' % meta['hostvars'][instance.name]['brook_status']
                if status_group in groups:
                    groups[status_group].append(instance.name)
                else:
                    groups[status_group] = [instance.name]

                # Group by tags
                tags = meta['hostvars'][instance.name]['brook_tags']
                for tag in tags:
                    tag_group = 'tag_%s' % tag
                    if tag_group in groups:
                        groups[tag_group].append(instance.name)
                    else:
                        groups[tag_group] = [instance.name]

        groups['_meta'] = meta
        return groups

    def hostvars(self, project, instance, template, api):
        """Return the hostvars dictionary for the given instance.

        Raise libbrook.rest.ApiException if it cannot retrieve all required information from the
        Brook.io API.
        """

        hostvars = instance.to_dict()
        hostvars['brook_pid'] = hostvars.pop('pid')
        hostvars['brook_name'] = hostvars.pop('name')
        hostvars['brook_description'] = hostvars.pop('description')
        hostvars['brook_project'] = hostvars.pop('project')
        hostvars['brook_template'] = hostvars.pop('template')
        hostvars['brook_region'] = hostvars.pop('region')
        hostvars['brook_zone'] = hostvars.pop('zone')
        hostvars['brook_created_at'] = hostvars.pop('created_at')
        hostvars['brook_updated_at'] = hostvars.pop('updated_at')
        del hostvars['id']
        del hostvars['key']
        del hostvars['provider']
        del hostvars['image']

        # Substitute identifiers for names
        #
        hostvars['brook_project'] = project.name
        hostvars['brook_template'] = template.name if template else None

        # Retrieve instance state
        #
        status = api.status_instance(project_id=project.id, instance_id=instance.id)
        hostvars.update({'brook_status': status.state})

        # Retrieve instance tags
        #
        tags = api.instance_tags(project_id=project.id, instance_id=instance.id)
        hostvars.update({'brook_tags': tags})

        # Retrieve instance addresses
        #
        addresses = api.instance_addresses(project_id=project.id, instance_id=instance.id)
        internal_ips = [address.address for address in addresses if address.scope == 'internal']
        external_ips = [address.address for address in addresses
                        if address.address and address.scope == 'external']
        hostvars.update({'brook_internal_ips': internal_ips})
        hostvars.update({'brook_external_ips': external_ips})
        try:
            hostvars.update({'ansible_ssh_host': external_ips[0]})
        except IndexError:
            raise libbrook.rest.ApiException(status='502', reason='Instance without public IP')

        return hostvars


# Run the script
#
brook = BrookInventory()
print(json.dumps(brook.inventory))



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = '''
---
module: ce_is_is_view
version_added: '0.2.0'
author: xuxiaowei0512 (@CloudEngine-Ansible)
short_description: Manages isis view configuration on HUAWEI CloudEngine devices.
description:
    - Manages isis process id, creates a isis instance id or deletes a process id
      on HUAWEI CloudEngine devices.
options:
  coststyle:
    description:
      - Specifies the cost style.
    type: str
    choices: ['narrow', 'wide', 'transition', 'ntransition', 'wtransition']
  cost_type:
    description:
      - Specifies the cost type.
    type: str
    choices: ['external', 'internal']
  defaultmode:
    description:
      - Specifies the default mode.
    type: str
    choices: ['always', 'matchDefault', 'matchAny']
  export_policytype:
    description:
      - Specifies the default mode.
    type: str
    choices: ['aclNumOrName', 'ipPrefix', 'routePolicy']
  export_protocol:
    description:
      - Specifies the export router protocol.
    type: str
    choices: ['direct', 'ospf', 'isis', 'static', 'rip', 'bgp', 'ospfv3', 'all']
  impotr_leveltype:
    description:
      - Specifies the export router protocol.
    type: str
    choices: ['level_1', 'level_2', 'level_1_2']
  islevel:
    description:
      - Specifies the isis level.
    type: str
    choices: ['level_1', 'level_2', 'level_1_2']
  level_type:
    description:
      - Specifies the isis level type.
    type: str
    choices: ['level_1', 'level_2', 'level_1_2']
  penetration_direct:
    description:
      - Specifies the penetration direct.
    type: str
    choices: ['level2-level1', 'level1-level2']
  protocol:
    description:
      - Specifies the protocol.
    type: str
    choices: ['direct', 'ospf', 'isis', 'static', 'rip', 'bgp', 'ospfv3', 'all']
  aclnum_or_name:
    description:
      - Specifies the acl number or name for isis.
    type: str
  allow_filter:
    description:
      - Specifies the alow filter or not.
    type: bool
  allow_up_down:
    description:
      - Specifies the alow up or down.
    type: bool
  autocostenable:
    description:
      - Specifies the alow auto cost enable.
    type: bool
  autocostenablecompatible:
    description:
      - Specifies the alow auto cost enable compatible.
    type: bool
  avoid_learning:
    description:
      - Specifies the alow avoid learning.
    type: bool
  bfd_min_tx:
    description:
      - Specifies the bfd min sent package.
    type: int
  bfd_min_rx:
    description:
      - Specifies the bfd min received package.
    type: int
  bfd_multiplier_num:
    description:
      - Specifies the bfd multiplier number.
    type: int
  cost:
    description:
      - Specifies the bfd cost.
    type: int
  description:
    description:
      - Specifies description of isis.
    type: str
  enablelevel1tolevel2:
    description:
      - Enable level1 to level2.
    type: bool
  export_aclnumorname:
    description:
      - Specifies export acl number or name.
    type: str
  export_ipprefix:
    description:
      - Specifies export ip prefix.
    type: str
  export_processid:
    description:
      - Specifies export process id.
    type: int
  export_routepolicyname:
    description:
      - Specifies export route policy name.
    type: str
  import_aclnumorname:
    description:
      - Specifies import acl number or name.
    type: str
  import_cost:
    description:
      - Specifies import cost.
    type: int
  import_ipprefix:
    description:
      - Specifies import ip prefix.
    type: str
  import_route_policy:
    description:
      - Specifies import route policy.
    type: str
  import_routepolicy_name:
    description:
      - Specifies import route policy name.
    type: str
  import_routepolicyname:
    description:
      - Specifies import route policy name.
    type: str
  import_tag:
    description:
      - Specifies import tag.
    type: int
  inheritcost:
    description:
      - Enable inherit cost.
    type: bool
  instance_id:
    description:
      - Specifies instance id.
    type: int
  ip_address:
    description:
      - Specifies ip address.
    type: str
  ip_prefix_name:
    description:
      - Specifies ip prefix name.
    type: str
  max_load:
    description:
      - Specifies route max load.
    type: int
  mode_routepolicyname:
    description:
      - Specifies the mode of route polic yname.
    type: str
  mode_tag:
    description:
      - Specifies the tag of mode.
    type: int
  netentity:
    description:
      - Specifies the netentity.
    type: str
  permitibgp:
    description:
      - Specifies the permitibgp.
    type: bool
  processid:
    description:
      - Specifies the process id.
    type: int
  relaxspfLimit:
    description:
      - Specifies enable the relax spf limit.
    type: bool
  route_policy_name:
    description:
      - Specifies the route policy name.
    type: str
  stdbandwidth:
    description:
      - Specifies the std band width.
    type: int
  stdlevel1cost:
    description:
      - Specifies the std level1 cost.
    type: int
  stdlevel2cost:
    description:
      - Specifies the std level2 cost.
    type: int
  tag:
    description:
      - Specifies the isis tag.
    type: int
  weight:
    description:
      - Specifies the isis weight.
    type: int
  preference_value:
    description:
      - Specifies the preference value.
    type: int
  state:
    description:
      - Determines whether the config should be present or not on the device.
    default: present
    type: str
    choices: ['present', 'absent']
notes:
  - This module requires the netconf system service be enabled on the remote device being managed.
  - This module works with connection C(netconf).
'''

EXAMPLES = '''
  - name: Set isis description
    community.network.ce_is_is_view:
      instance_id: 3
      description: abcdeggfs
      state: present

  - name: Set isis islevel
    community.network.ce_is_is_view:
      instance_id: 3
      islevel: level_1
      state: present
  - name: Set isis coststyle
    community.network.ce_is_is_view:
      instance_id: 3
      coststyle: narrow
      state: present

  - name: Set isis stdlevel1cost
    community.network.ce_is_is_view:
      instance_id: 3
      stdlevel1cost: 63
      state: present

  - name: Set isis stdlevel2cost
    community.network.ce_is_is_view:
      instance_id: 3
      stdlevel2cost: 63
      state: present

  - name: Set isis stdbandwidth
    community.network.ce_is_is_view:
      instance_id: 3
      stdbandwidth: 1
      state: present
'''

RETURN = '''
proposed:
    description: k/v pairs of parameters passed into module
    returned: always
    type: dict
    sample: {
        "state": "present"
    }
existing:
    description: k/v pairs of existing configuration
    returned: always
    type: dict
    sample: {
        "session": {}
    }
end_state:
    description: k/v pairs of configuration after module execution
    returned: always
    type: dict
    sample: {
        "session": {
            "addrType": "IPV4",
            "createType": "SESS_STATIC",
            "destAddr": null,
            "outIfName": "10GE1/0/1",
            "sessName": "bfd_l2link",
            "srcAddr": null,
            "useDefaultIp": "true",
            "vrfName": null
        }
    }
updates:
    description: commands sent to the device
    returned: always
    type: list
    sample: [
        "bfd bfd_l2link bind peer-ip default-ip interface 10ge1/0/1"
    ]
changed:
    description: check to see if a change was made on the device
    returned: always
    type: bool
    sample: true
'''

import sys
import socket
from xml.etree import ElementTree
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.network.plugins.module_utils.network.cloudengine.ce import get_nc_config, set_nc_config

CE_NC_GET_ISIS = """
    <filter type="subtree">
      <isiscomm xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
      %s
      </isiscomm>
    </filter>
"""

CE_NC_GET_ISIS_INSTANCE = """
        <isSites>
          <isSite>
            <instanceId>%s</instanceId>
            <description></description>
            <isLevel></isLevel>
            <costStyle></costStyle>
            <relaxSpfLimit></relaxSpfLimit>
            <stdLevel1Cost></stdLevel1Cost>
            <stdLevel2Cost></stdLevel2Cost>
            <stdbandwidth></stdbandwidth>
            <stdAutoCostEnable></stdAutoCostEnable>
            <stdAutoCostEnableCompatible></stdAutoCostEnableCompatible>
          </isSite>
        </isSites>
"""

CE_NC_GET_ISIS_ENTITY = """
        <isSites>
          <isSite>
            <instanceId>%s</instanceId>
            <isNetEntitys>
              <isNetEntity>
                <netEntity></netEntity>
              </isNetEntity>
            </isNetEntitys>
          </isSite>
        </isSites>
"""

CE_NC_CREAT_ISIS_ENTITY = """
        <isSites>
          <isSite>
            <instanceId>%s</instanceId>
            <isNetEntitys>
              <isNetEntity operation="merge">
                <netEntity>%s</netEntity>
              </isNetEntity>
            </isNetEntitys>
          </isSite>
        </isSites>
"""

CE_NC_DELATE_ISIS_ENTITY = """
        <isSites>
          <isSite>
            <instanceId>%s</instanceId>
            <isNetEntitys>
              <isNetEntity operation="delete">
                <netEntity>%s</netEntity>
              </isNetEntity>
            </isNetEntitys>
          </isSite>
        </isSites>
"""

CE_NC_GET_ISIS_PREFERENCE = """
        <isSites>
          <isSite>
            <instanceId>%s</instanceId>
            <isSiteMTs>
              <isSiteMT>
                <isPreferences>
                  <isPreference>
                    <preferenceValue></preferenceValue>
                    <routePolicyName></routePolicyName>
                  </isPreference>
                </isPreferences>
              </isSiteMT>
            </isSiteMTs>
          </isSite>
        </isSites>
"""

CE_NC_MREGE_ISIS_PREFERENCE = """
        <isSites>
          <isSite>
            <instanceId>%s</instanceId>
            <isSiteMTs>
              <isSiteMT>
                <addressFamily>afIpv4</addressFamily>
                <mtId>0</mtId>
                <isPreferences>
                  <isPreference operation="merge">
                  %s
                  </isPreference>
                </isPreferences>
              </isSiteMT>
            </isSiteMTs>
          </isSite>
        </isSites>
"""

CE_NC_DELETE_ISIS_PREFERENCE = """
        <isSites>
          <isSite>
            <instanceId>%s</instanceId>
            <isSiteMTs>
              <isSiteMT>
                <addressFamily>afIpv4</addressFamily>
                <mtId>0</mtId>
                <isPreferences>
                  <isPreference operation="delete">
                  %s
                  </isPreference>
                </isPreferences>
              </isSiteMT>
            </isSiteMTs>
          </isSite>
        </isSites>
"""

CE_NC_GET_ISIS_MAXLOAD = """
        <isSites>
          <isSite>
            <instanceId>%s</instanceId>
            <isSiteMTs>
              <isSiteMT>
                <addressFamily>afIpv4</addressFamily>
                <mtId>0</mtId>
                <maxLoadBalancing></maxLoadBalancing>
              </isSiteMT>
            </isSiteMTs>
          </isSite>
        </isSites>
"""

CE_NC_MERGE_ISIS_MAXLOAD = """
        <isSites>
          <isSite>
            <instanceId>%s</instanceId>
            <isSiteMTs>
              <isSiteMT operation="merge">
                <addressFamily>afIpv4</addressFamily>
                <mtId>0</mtId>
                <maxLoadBalancing>%s</maxLoadBalancing>
              </isSiteMT>
            </isSiteMTs>
          </isSite>
        </isSites>
"""

CE_NC_DELETE_ISIS_MAXLOAD = """
        <isSites>
          <isSite>
            <instanceId>%s</instanceId>
            <isSiteMTs>
              <isSiteMT operation="delete">
                <addressFamily>afIpv4</addressFamily>
                <mtId>0</mtId>
                <maxLoadBalancing>32</maxLoadBalancing>
              </isSiteMT>
            </isSiteMTs>
          </isSite>
        </isSites>
"""

CE_NC_GET_ISIS_NEXTHOP = """
        <isSites>
          <isSite>
            <instanceId>%s</instanceId>
            <isSiteMTs>
              <isSiteMT>
                <addressFamily>afIpv4</addressFamily>
                <mtId>0</mtId>
                <isNextHopWeights>
                  <isNextHopWeight>
                    <ipAddress></ipAddress>
                    <weight></weight>
                  </isNextHopWeight>
                </isNextHopWeights>
              </isSiteMT>
            </isSiteMTs>
          </isSite>
        </isSites>
"""

CE_NC_MERGE_ISIS_NEXTHOP = """
        <isSites>
          <isSite>
            <instanceId>%s</instanceId>
            <isSiteMTs>
              <isSiteMT>
                <addressFamily>afIpv4</addressFamily>
                <mtId>0</mtId>
                <isNextHopWeights>
                  <isNextHopWeight operation="merge">
                    <ipAddress>%s</ipAddress>
                    <weight>%s</weight>
                  </isNextHopWeight>
                </isNextHopWeights>
              </isSiteMT>
            </isSiteMTs>
          </isSite>
        </isSites>
"""

CE_NC_DELETE_ISIS_NEXTHOP = """
        <isSites>
          <isSite>
            <instanceId>%s</instanceId>
            <isSiteMTs>
              <isSiteMT>
                <addressFamily>afIpv4</addressFamily>
                <mtId>0</mtId>
                <isNextHopWeights>
                  <isNextHopWeight operation="delete">
                    <ipAddress>%s</ipAddress>
                    <weight>1</weight>
                  </isNextHopWeight>
                </isNextHopWeights>
              </isSiteMT>
            </isSiteMTs>
          </isSite>
        </isSites>
"""

CE_NC_GET_ISIS_LEAKROUTELEVEL2 = """
        <isSites>
          <isSite>
            <instanceId>%s</instanceId>
            <isSiteMTs>
              <isSiteMT>
                <addressFamily>afIpv4</addressFamily>
                <mtId>0</mtId>
                <isLeakRouteLevel2ToLevel1s>
                  <isLeakRouteLevel2ToLevel1>
                    <tag></tag>
                    <routePolicyName></routePolicyName>
                    <aclNumOrName></aclNumOrName>
                    <ipPrefix></ipPrefix>
                    <allowFilter></allowFilter>
                    <allowUpdown></allowUpdown>
                  </isLeakRouteLevel2ToLevel1>
                </isLeakRouteLevel2ToLevel1s>
              </isSiteMT>
            </isSiteMTs>
          </isSite>
        </isSites>
"""

CE_NC_MERGE_ISIS_LEAKROUTELEVEL2 = """
        <isSites>
          <isSite>
            <instanceId>%s</instanceId>
            <isSiteMTs>
              <isSiteMT>
                <addressFamily>afIpv4</addressFamily>
                <mtId>0</mtId>
                <isLeakRouteLevel2ToLevel1s>
                  <isLeakRouteLevel2ToLevel1 operation="merge">
                   %s
                  </isLeakRouteLevel2ToLevel1>
                </isLeakRouteLevel2ToLevel1s>
              </isSiteMT>
            </isSiteMTs>
          </isSite>
        </isSites>
"""

CE_NC_DELETE_ISIS_LEAKROUTELEVEL2 = """
        <isSites>
          <isSite>
            <instanceId>%s</instanceId>
            <isSiteMTs>
              <isSiteMT>
                <addressFamily>afIpv4</addressFamily>
                <mtId>0</mtId>
                <isLeakRouteLevel2ToLevel1s>
                  <isLeakRouteLevel2ToLevel1 operation="delete">
                    <tag>0</tag>
                    <routePolicyName/>
                    <aclNumOrName/>
                    <ipPrefix/>
                    <allowFilter>false</allowFilter>
                  </isLeakRouteLevel2ToLevel1>
                </isLeakRouteLevel2ToLevel1s>
              </isSiteMT>
            </isSiteMTs>
          </isSite>
        </isSites>
"""

CE_NC_GET_ISIS_LEAKROUTELEVEL1 = """
        <isSites>
          <isSite>
            <instanceId>%s</instanceId>
            <isSiteMTs>
              <isSiteMT>
                <addressFamily>afIpv4</addressFamily>
                <mtId>0</mtId>
                <isLeakRouteLevel1ToLevel2s>
                  <isLeakRouteLevel1ToLevel2>
                    <tag></tag>
                    <routePolicyName></routePolicyName>
                    <aclNumOrName></aclNumOrName>
                    <ipPrefix></ipPrefix>
                    <leakEnableFlag></leakEnableFlag>
                    <allowFilter></allowFilter>
                  </isLeakRouteLevel1ToLevel2>
                </isLeakRouteLevel1ToLevel2s>
              </isSiteMT>
            </isSiteMTs>
          </isSite>
        </isSites>
"""

CE_NC_MERGE_ISIS_LEAKROUTELEVEL1 = """
        <isSites>
          <isSite>
            <instanceId>%s</instanceId>
            <isSiteMTs>
              <isSiteMT>
                <addressFamily>afIpv4</addressFamily>
                <mtId>0</mtId>
                <isLeakRouteLevel1ToLevel2s>
                  <isLeakRouteLevel1ToLevel2 operation="merge">
                  %s
                  </isLeakRouteLevel1ToLevel2>
                </isLeakRouteLevel1ToLevel2s>
              </isSiteMT>
            </isSiteMTs>
          </isSite>
        </isSites>
"""

CE_NC_DELETE_ISIS_LEAKROUTELEVEL1 = """
        <isSites>
          <isSite>
            <instanceId>%s</instanceId>
            <isSiteMTs>
              <isSiteMT>
                <addressFamily>afIpv4</addressFamily>
                <mtId>0</mtId>
                <isLeakRouteLevel1ToLevel2s>
                  <isLeakRouteLevel1ToLevel2 operation="delete">
                    <tag>0</tag>
                    <routePolicyName/>
                    <aclNumOrName/>
                    <ipPrefix/>
                    <leakEnableFlag>false</leakEnableFlag>
                    <allowFilter>false</allowFilter>
                  </isLeakRouteLevel1ToLevel2>
                </isLeakRouteLevel1ToLevel2s>
              </isSiteMT>
            </isSiteMTs>
          </isSite>
        </isSites>
"""

CE_NC_GET_ISIS_DEFAULTROUTE = """
        <isSites>
          <isSite>
            <instanceId>%s</instanceId>
            <isSiteMTs>
              <isSiteMT>
                <addressFamily>afIpv4</addressFamily>
                <mtId>0</mtId>
                <isDefaultRoutes>
                  <isDefaultRoute>
                    <defaultMode></defaultMode>
                    <routePolicyName></routePolicyName>
                    <cost></cost>
                    <tag></tag>
                    <levelType></levelType>
                    <avoidLearning></avoidLearning>
                  </isDefaultRoute>
                </isDefaultRoutes>
              </isSiteMT>
            </isSiteMTs>
          </isSite>
        </isSites>
"""

CE_NC_MERGE_ISIS_DEFAULTROUTE = """
        <isSites>
          <isSite>
            <instanceId>%s</instanceId>
            <isSiteMTs>
              <isSiteMT>
                <addressFamily>afIpv4</addressFamily>
                <mtId>0</mtId>
                <isDefaultRoutes>
                  <isDefaultRoute operation="merge">
                   %s
                  </isDefaultRoute>
                </isDefaultRoutes>
              </isSiteMT>
            </isSiteMTs>
          </isSite>
        </isSites>
"""

CE_NC_DELETE_ISIS_DEFAULTROUTE = """
        <isSites>
          <isSite>
            <instanceId>%s</instanceId>
            <isSiteMTs>
              <isSiteMT>
                <addressFamily>afIpv4</addressFamily>
                <mtId>0</mtId>
                <isDefaultRoutes>
                  <isDefaultRoute operation="delete">
                    <defaultMode>always</defaultMode>
                    <cost>0</cost>
                    <tag>0</tag>
                    <levelType>level_2</levelType>
                    <avoidLearning>false</avoidLearning>
                  </isDefaultRoute>
                </isDefaultRoutes>
              </isSiteMT>
            </isSiteMTs>
          </isSite>
        </isSites>
"""

CE_NC_GET_ISIS_IMPORTROUTE = """
        <isSites>
          <isSite>
            <instanceId>%s</instanceId>
            <isSiteMTs>
              <isSiteMT>
                <addressFamily>afIpv4</addressFamily>
                <mtId>0</mtId>
                <isImportRoutes>
                  <isImportRoute>
                    <protocol></protocol>
                    <processId></processId>
                    <costType></costType>
                    <cost></cost>
                    <tag></tag>
                    <policyType></policyType>
                    <routePolicyName></routePolicyName>
                    <levelType></levelType>
                    <inheritCost></inheritCost>
                    <permitIbgp></permitIbgp>
                  </isImportRoute>
                </isImportRoutes>
              </isSiteMT>
            </isSiteMTs>
          </isSite>
        </isSites>
"""

CE_NC_MERGE_ISIS_IMPORTROUTE = """
        <isSites>
          <isSite>
            <instanceId>%s</instanceId>
            <isSiteMTs>
              <isSiteMT>
                <addressFamily>afIpv4</addressFamily>
                <mtId>0</mtId>
                <isImportRoutes>
                  <isImportRoute operation="merge">
                   %s
                  </isImportRoute>
                </isImportRoutes>
              </isSiteMT>
            </isSiteMTs>
          </isSite>
        </isSites>
"""

CE_NC_GET_ISIS_EXPORTROUTE = """
        <isSites>
          <isSite>
            <instanceId>%s</instanceId>
            <isSiteMTs>
              <isSiteMT>
                <addressFamily>afIpv4</addressFamily>
                <mtId>0</mtId>
                <isFilterExports>
                  <isFilterExport>
                    <protocol></protocol>
                    <processId></processId>
                    <policyType></policyType>
                  </isFilterExport>
                </isFilterExports>
              </isSiteMT>
            </isSiteMTs>
          </isSite>
        </isSites>
"""

CE_NC_MERGE_ISIS_EXPORTROUTE = """
        <isSites>
          <isSite>
            <instanceId>%s</instanceId>
            <isSiteMTs>
              <isSiteMT>
                <addressFamily>afIpv4</addressFamily>
                <mtId>0</mtId>
                <isFilterExports>
                  <isFilterExport operation="merge">
                  %s
                  </isFilterExport>
                </isFilterExports>
              </isSiteMT>
            </isSiteMTs>
          </isSite>
        </isSites>
"""

CE_NC_GET_ISIS_IMPORTIPROUTE = """
        <isSites>
          <isSite>
            <instanceId>%s</instanceId>
            <isSiteMTs>
              <isSiteMT>
                <addressFamily>afIpv4</addressFamily>
                <mtId>0</mtId>
                <isFilterImports>
                  <isFilterImport>
                    <aclNumOrName></aclNumOrName>
                    <ipPrefix></ipPrefix>
                    <routePolicyName></routePolicyName>
                    <policyType></policyType>
                  </isFilterImport>
                </isFilterImports>
              </isSiteMT>
            </isSiteMTs>
          </isSite>
        </isSites>
"""

CE_NC_MERGE_ISIS_IMPORTIPROUTE = """
        <isSites>
          <isSite>
            <instanceId>%s</instanceId>
            <isSiteMTs>
              <isSiteMT>
                <addressFamily>afIpv4</addressFamily>
                <mtId>0</mtId>
                <isFilterImports>
                  <isFilterImport operation="merge">
                   %s
                  </isFilterImport>
                </isFilterImports>
              </isSiteMT>
            </isSiteMTs>
          </isSite>
        </isSites>
"""

CE_NC_GET_ISIS_BFDLINK = """
        <isSites>
          <isSite>
            <instanceId>%s</instanceId>
            <isSiteMTs>
              <isSiteMT>
                <addressFamily>afIpv4</addressFamily>
                <mtId>0</mtId>
                <bfdMinRx></bfdMinRx>
                <bfdMinTx></bfdMinTx>
                <bfdMultNum></bfdMultNum>
              </isSiteMT>
            </isSiteMTs>
          </isSite>
        </isSites>
"""

CE_NC_MERGE_ISIS_BFDLINK = """
        <isSites>
          <isSite>
            <instanceId>%s</instanceId>
            <isSiteMTs>
              <isSiteMT operation="merge">
                <addressFamily>afIpv4</addressFamily>
                <mtId>0</mtId>
                %s
              </isSiteMT>
            </isSiteMTs>
          </isSite>
        </isSites>
"""

CE_NC_DELETE_ISIS_BFDLINK = """
        <isSites>
          <isSite>
            <instanceId>%s</instanceId>
            <isSiteMTs>
              <isSiteMT operation="delete">
                <addressFamily>afIpv4</addressFamily>
                <mtId>0</mtId>
                <bfdMinRx>3</bfdMinRx>
                <bfdMinTx>3</bfdMinTx>
                <bfdMultNum>3</bfdMultNum>
              </isSiteMT>
            </isSiteMTs>
          </isSite>
        </isSites>
"""


def is_valid_ip_vpn(vpname):
    """check ip vpn"""

    if not vpname:
        return False

    if vpname == "_public_":
        return False

    if len(vpname) < 1 or len(vpname) > 31:
        return False

    return True


def check_ip_addr(ipaddr):
    """check ip address, Supports IPv4 and IPv6"""

    if not ipaddr or '\x00' in ipaddr:
        return False

    try:
        res = socket.getaddrinfo(ipaddr, 0, socket.AF_UNSPEC,
                                 socket.SOCK_STREAM,
                                 0, socket.AI_NUMERICHOST)
        return bool(res)
    except socket.gaierror:
        err = sys.exc_info()[1]
        if err.args[0] == socket.EAI_NONAME:
            return False
        raise

    return True


def check_default_ip(ipaddr):
    """check the default multicast IP address"""

    # The value ranges from 224.0.0.107 to 224.0.0.250
    if not check_ip_addr(ipaddr):
        return False

    if ipaddr.count(".") != 3:
        return False

    ips = ipaddr.split(".")
    if ips[0] != "224" or ips[1] != "0" or ips[2] != "0":
        return False

    if not ips[3].isdigit() or int(ips[3]) < 107 or int(ips[3]) > 250:
        return False

    return True


def get_interface_type(interface):
    """get the type of interface, such as 10GE, ETH-TRUNK, VLANIF..."""

    if interface is None:
        return None

    if interface.upper().startswith('GE'):
        iftype = 'ge'
    elif interface.upper().startswith('10GE'):
        iftype = '10ge'
    elif interface.upper().startswith('25GE'):
        iftype = '25ge'
    elif interface.upper().startswith('4X10GE'):
        iftype = '4x10ge'
    elif interface.upper().startswith('40GE'):
        iftype = '40ge'
    elif interface.upper().startswith('100GE'):
        iftype = '100ge'
    elif interface.upper().startswith('VLANIF'):
        iftype = 'vlanif'
    elif interface.upper().startswith('LOOPBACK'):
        iftype = 'loopback'
    elif interface.upper().startswith('METH'):
        iftype = 'meth'
    elif interface.upper().startswith('ETH-TRUNK'):
        iftype = 'eth-trunk'
    elif interface.upper().startswith('VBDIF'):
        iftype = 'vbdif'
    elif interface.upper().startswith('NVE'):
        iftype = 'nve'
    elif interface.upper().startswith('TUNNEL'):
        iftype = 'tunnel'
    elif interface.upper().startswith('ETHERNET'):
        iftype = 'ethernet'
    elif interface.upper().startswith('FCOE-PORT'):
        iftype = 'fcoe-port'
    elif interface.upper().startswith('FABRIC-PORT'):
        iftype = 'fabric-port'
    elif interface.upper().startswith('STACK-PORT'):
        iftype = 'stack-port'
    elif interface.upper().startswith('NULL'):
        iftype = 'null'
    else:
        return None

    return iftype.lower()


class ISIS_View(object):
    """Manages ISIS Instance"""

    def __init__(self, argument_spec):
        self.spec = argument_spec
        self.module = None
        self.__init_module__()

        # module input info
        self.instance_id = self.module.params['instance_id']
        self.description = self.module.params['description']
        self.islevel = self.module.params['islevel']
        self.coststyle = self.module.params['coststyle']
        self.relaxspfLimit = self.module.params['relaxspfLimit']
        self.stdlevel1cost = self.module.params['stdlevel1cost']
        self.stdlevel2cost = self.module.params['stdlevel2cost']
        self.stdbandwidth = self.module.params['stdbandwidth']
        self.autocostenable = self.module.params['autocostenable']
        self.autocostenablecompatible = self.module.params['autocostenablecompatible']
        self.netentity = self.module.params['netentity']
        self.preference_value = self.module.params['preference_value']
        self.route_policy_name = self.module.params['route_policy_name']
        self.max_load = self.module.params['max_load']
        self.ip_address = self.module.params['ip_address']
        self.weight = self.module.params['weight']
        self.aclnum_or_name = self.module.params['aclnum_or_name']
        self.ip_prefix_name = self.module.params['ip_prefix_name']
        self.import_routepolicy_name = self.module.params['import_routepolicy_name']
        self.tag = self.module.params['tag']
        self.allow_filter = self.module.params['allow_filter']
        self.allow_up_down = self.module.params['allow_up_down']
        self.penetration_direct = self.module.params['penetration_direct']
        self.enablelevel1tolevel2 = self.module.params['enablelevel1tolevel2']
        self.defaultmode = self.module.params['defaultmode']
        self.mode_routepolicyname = self.module.params['mode_routepolicyname']
        self.cost = self.module.params['cost']
        self.mode_tag = self.module.params['mode_tag']
        self.level_type = self.module.params['level_type']
        self.avoid_learning = self.module.params['avoid_learning']
        self.protocol = self.module.params['protocol']
        self.processid = self.module.params['processid']
        self.cost_type = self.module.params['cost_type']
        self.import_cost = self.module.params['import_cost']
        self.import_tag = self.module.params['import_tag']
        self.impotr_leveltype = self.module.params['impotr_leveltype']
        self.import_route_policy = self.module.params['import_route_policy']
        self.inheritcost = self.module.params['inheritcost']
        self.permitibgp = self.module.params['permitibgp']
        self.avoid_learning = self.module.params['avoid_learning']
        self.export_protocol = self.module.params['export_protocol']
        self.export_policytype = self.module.params['export_policytype']
        self.export_processid = self.module.params['export_processid']
        self.export_aclnumorname = self.module.params['export_aclnumorname']
        self.export_ipprefix = self.module.params['export_ipprefix']
        self.export_routepolicyname = self.module.params['export_routepolicyname']
        self.import_aclnumorname = self.module.params['import_aclnumorname']
        self.import_ipprefix = self.module.params['import_ipprefix']
        self.import_routepolicyname = self.module.params['import_routepolicyname']
        self.bfd_min_rx = self.module.params['bfd_min_rx']
        self.bfd_min_tx = self.module.params['bfd_min_tx']
        self.bfd_multiplier_num = self.module.params['bfd_multiplier_num']
        self.state = self.module.params['state']

        # state
        self.changed = False
        self.isis_dict = dict()
        self.updates_cmd = list()
        self.commands = list()
        self.results = dict()
        self.proposed = dict()
        self.existing = dict()
        self.end_state = dict()

    def __init_module__(self):
        """init module"""

        mutually_exclusive = [["stdlevel1cost", "stdlevel2cost"],
                              ["aclnum_or_name", "ip_prefix_name", "import_routepolicy_name"],
                              ["export_aclnumorname", "import_ipprefix", "import_routepolicyname"]]
        required_together = [('ip_address', 'weight')]
        self.module = AnsibleModule(
            argument_spec=self.spec,
            mutually_exclusive=mutually_exclusive,
            required_together=required_together,
            supports_check_mode=True)

    def get_isis_dict(self):
        """bfd config dict"""

        isis_dict = dict()
        isis_dict["instance"] = dict()
        conf_str = CE_NC_GET_ISIS % (
            (CE_NC_GET_ISIS_INSTANCE % self.instance_id))

        if self.netentity:
            conf_str = CE_NC_GET_ISIS % (
                (CE_NC_GET_ISIS_ENTITY % self.instance_id))

        if self.route_policy_name or self.preference_value:
            conf_str = CE_NC_GET_ISIS % (
                (CE_NC_GET_ISIS_PREFERENCE % self.instance_id))
        if self.max_load:
            conf_str = CE_NC_GET_ISIS % (
                (CE_NC_GET_ISIS_MAXLOAD % self.instance_id))
        if self.ip_address:
            conf_str = CE_NC_GET_ISIS % (
                (CE_NC_GET_ISIS_NEXTHOP % self.instance_id))
        if self.penetration_direct and self.penetration_direct == "level2-level1":
            conf_str = CE_NC_GET_ISIS % (
                (CE_NC_GET_ISIS_LEAKROUTELEVEL2 % self.instance_id))
        elif self.penetration_direct and self.penetration_direct == "level1-level2":
            conf_str = CE_NC_GET_ISIS % (
                (CE_NC_GET_ISIS_LEAKROUTELEVEL1 % self.instance_id))
        elif self.defaultmode:
            conf_str = CE_NC_GET_ISIS % (
                (CE_NC_GET_ISIS_DEFAULTROUTE % self.instance_id))
        elif self.protocol:
            conf_str = CE_NC_GET_ISIS % (
                (CE_NC_GET_ISIS_IMPORTROUTE % self.instance_id))
        elif self.export_protocol:
            conf_str = CE_NC_GET_ISIS % (
                (CE_NC_GET_ISIS_EXPORTROUTE % self.instance_id))
        elif self.bfd_min_rx or self.bfd_min_tx or self.bfd_multiplier_num:
            conf_str = CE_NC_GET_ISIS % (
                (CE_NC_GET_ISIS_BFDLINK % self.instance_id))
        elif self.import_aclnumorname or self.import_ipprefix or self.import_ipprefix:
            conf_str = CE_NC_GET_ISIS % (
                (CE_NC_GET_ISIS_IMPORTIPROUTE % self.instance_id))
        xml_str = get_nc_config(self.module, conf_str)

        if "<data/>" in xml_str:
            return isis_dict

        xml_str = xml_str.replace('\r', '').replace('\n', '').\
            replace('xmlns="urn:ietf:params:xml:ns:netconf:base:1.0"', "").\
            replace('xmlns="http://www.huawei.com/netconf/vrp"', "")
        root = ElementTree.fromstring(xml_str)

        # get bfd global info
        if self.netentity:
            glb = root.find("isiscomm/isSites/isSite/isNetEntitys/isNetEntity")
        elif self.route_policy_name or self.preference_value:
            glb = root.find("isiscomm/isSites/isSite//isSiteMTs/isSiteMT/isPreferences/isPreference")
        elif self.max_load:
            glb = root.find("isiscomm/isSites/isSite/isSiteMTs/isSiteMT")
        elif self.ip_address:
            glb = root.find("isiscomm/isSites/isSite/isSiteMTs/isSiteMT/isNextHopWeights/isNextHopWeight")
        elif self.penetration_direct and self.penetration_direct == "level2-level1":
            glb = root.find("isiscomm/isSites/isSite/isSiteMTs/isSiteMT/isLeakRouteLevel2ToLevel1s/isLeakRouteLevel2ToLevel1")
        elif self.penetration_direct and self.penetration_direct == "level1-level2":
            glb = root.find(
                "isiscomm/isSites/isSite/isSiteMTs/isSiteMT/isLeakRouteLevel1ToLevel2s/isLeakRouteLevel1ToLevel2")
        elif self.defaultmode:
            glb = root.find(
                "isiscomm/isSites/isSite/isSiteMTs/isSiteMT/isDefaultRoutes/isDefaultRoute")
        elif self.protocol:
            glb = root.find(
                "isiscomm/isSites/isSite/isSiteMTs/isSiteMT/isImportRoutes/isImportRoute")
        elif self.export_protocol:
            glb = root.find(
                "isiscomm/isSites/isSite/isSiteMTs/isSiteMT/isFilterExports/isFilterExport")
        elif self.bfd_min_rx or self.bfd_min_tx or self.bfd_multiplier_num:
            glb = root.find(
                "isiscomm/isSites/isSite/isSiteMTs/isSiteMT")
        elif self.import_aclnumorname or self.import_ipprefix or self.import_ipprefix:
            glb = root.find(
                "isiscomm/isSites/isSite/isSiteMTs/isSiteMT/isFilterImports/isFilterImport")
        else:
            glb = root.find("isiscomm/isSites/isSite")

        if glb is not None:
            for attr in glb:
                isis_dict["instance"][attr.tag] = attr.text

        return isis_dict

    def config_session(self):
        """configures bfd session"""

        xml_str = ""
        instance = self.isis_dict["instance"]
        if not self.instance_id:
            return xml_str
        xml_str = "<instanceId>%s</instanceId>" % self.instance_id
        self.updates_cmd.append("isis %s" % self.instance_id)
        cmd_list = list()

        if self.state == "present":
            if self.description and self.description != instance.get("description"):
                xml_str += "<description>%s</description>" % self.description
                self.updates_cmd.append("description %s" % self.description)

            if self.islevel and self.islevel != instance.get("isLevel"):
                xml_str += "<isLevel>%s</isLevel>" % self.islevel
                self.updates_cmd.append("is-level %s" % self.islevel)

            if self.coststyle:
                if self.coststyle != instance.get("costStyle"):
                    xml_str += "<costStyle>%s</costStyle>" % self.coststyle
                    self.updates_cmd.append("cost-style %s" % self.coststyle)
                if self.relaxspfLimit and instance.get("relaxSpfLimit", "false") == "false":
                    xml_str += "<relaxSpfLimit>true</relaxSpfLimit>"
                    self.updates_cmd.append("cost-style %s relax-spf-limit" % self.coststyle)
                elif not self.relaxspfLimit and instance.get("relaxSpfLimit", "false") == "true":
                    xml_str += "<relaxSpfLimit>false</relaxSpfLimit>"
                    self.updates_cmd.append("cost-style %s" % self.coststyle)

            if self.stdlevel1cost and str(self.stdlevel1cost) != instance.get("stdLevel1Cost"):
                xml_str += "<stdLevel1Cost>%s</stdLevel1Cost>" % self.stdlevel1cost
                self.updates_cmd.append("circuit-cost %s level-1" % self.stdlevel1cost)

            if self.stdlevel2cost and str(self.stdlevel2cost) != instance.get("stdLevel2Cost"):
                xml_str += "<stdLevel2Cost>%s</stdLevel2Cost>" % self.stdlevel2cost
                self.updates_cmd.append("circuit-cost %s level-2" % self.stdlevel2cost)

            if self.stdbandwidth and str(self.stdbandwidth) != instance.get("stdbandwidth"):
                xml_str += "<stdbandwidth>%s</stdbandwidth>" % self.stdbandwidth
                self.updates_cmd.append("bandwidth-reference %s" % self.stdbandwidth)

            if self.netentity and self.netentity != instance.get("netEntity"):
                xml_str = CE_NC_CREAT_ISIS_ENTITY % (self.instance_id, self.netentity)
                self.updates_cmd.append("network-entity %s" % self.netentity)

            if self.preference_value or self.route_policy_name:
                xml_str = ""
                cmd_session = "preference"
                if self.preference_value and str(self.preference_value) != instance.get("preferenceValue"):
                    xml_str = "<preferenceValue>%s</preferenceValue>" % self.preference_value
                    cmd_session += " %s" % self.preference_value
                if self.route_policy_name and self.route_policy_name != instance.get("routePolicyName"):
                    xml_str += "<routePolicyName>%s</routePolicyName>" % self.route_policy_name
                    cmd_session += " route-policy %s" % self.route_policy_name
                cmd_list.insert(0, cmd_session)
                self.updates_cmd.extend(cmd_list)
                xml_str = CE_NC_MREGE_ISIS_PREFERENCE % (self.instance_id, xml_str)

            if self.max_load and str(self.max_load) != instance.get("maxLoadBalancing"):
                xml_str = CE_NC_MERGE_ISIS_MAXLOAD % (self.instance_id, self.max_load)
                self.updates_cmd.append("maximum load-balancing %s" % self.max_load)

            if self.ip_address:
                xml_str = CE_NC_MERGE_ISIS_NEXTHOP % (self.instance_id, self.ip_address, self.weight)
                self.updates_cmd.append("nexthop %s weight %s" % (self.ip_address, self.weight))

            if self.penetration_direct:
                xml_str = ""
                if self.penetration_direct == "level2-level1":
                    cmd_session = "import-route isis level-2 into level-1"
                elif self.penetration_direct == "level1-level2":
                    cmd_session = "import-route isis level-1 into level-2"
                if self.aclnum_or_name:
                    xml_str = "<aclNumOrName>%s</aclNumOrName>" % self.aclnum_or_name
                    xml_str += "<policyType>aclNumOrName</policyType>"
                    if isinstance(self.aclnum_or_name, int):
                        cmd_session += " filter-policy %s" % self.aclnum_or_name
                    elif isinstance(self.aclnum_or_name, str):
                        cmd_session += " filter-policy acl-name %s" % self.aclnum_or_name
                if self.ip_prefix_name:
                    xml_str = "<ipPrefix>%s</ipPrefix>" % self.ip_prefix_name
                    xml_str += "<policyType>ipPrefix</policyType>"
                    cmd_session += " filter-policy ip-prefix %s" % self.ip_prefix_name
                if self.import_routepolicy_name:
                    xml_str = "<routePolicyName>%s</routePolicyName>" % self.import_routepolicy_name
                    xml_str += "<policyType>routePolicy</policyType>"
                    cmd_session += " filter-policy route-policy %s" % self.import_routepolicy_name
                if self.tag:
                    xml_str += "<tag>%s</tag>" % self.tag
                    cmd_session += " tag %s" % self.tag
                if self.allow_filter or self.allow_up_down:
                    cmd_session += " direct"
                    if self.allow_filter:
                        xml_str += "<allowFilter>true</allowFilter>"
                        cmd_session += " allow-filter-policy"
                    if self.allow_up_down:
                        xml_str += "<allowUpdown>true</allowUpdown>"
                        cmd_session += " allow-up-down-bit"
                cmd_list.insert(0, cmd_session)
                self.updates_cmd.extend(cmd_list)
                if self.enablelevel1tolevel2:
                    xml_str += "<leakEnableFlag>true</leakEnableFlag>"
                    self.updates_cmd.append("undo import-route isis level-1 into level-2 disable")

            if self.defaultmode:
                cmd_session = "default-route-advertise"
                if self.defaultmode == "always":
                    xml_str = "<defaultMode>always</defaultMode>"
                    cmd_session += " always"
                elif self.defaultmode == "matchDefault":
                    xml_str = "<defaultMode>matchDefault</defaultMode>"
                    cmd_session += " match default"
                elif self.defaultmode == "matchAny":
                    xml_str = "<defaultMode>matchAny</defaultMode>"
                    xml_str += "<policyType>routePolicy</policyType>"
                    xml_str += "<routePolicyName>%s</routePolicyName>" % self.mode_routepolicyname
                    cmd_session += " route-policy %s" % self.mode_routepolicyname
                if self.cost is not None:
                    xml_str += "<cost>%s</cost>" % self.cost
                    cmd_session += " cost %s" % self.cost
                if self.mode_tag:
                    xml_str += "<tag>%s</tag>" % self.mode_tag
                    cmd_session += " tag %s" % self.mode_tag
                if self.level_type:
                    if self.level_type == "level_1":
                        xml_str += "<levelType>level_1</levelType>"
                        cmd_session += " level-1"
                    elif self.level_type == "level_2":
                        xml_str += "<levelType>level_2</levelType>"
                        cmd_session += " level-2"
                    elif self.level_type == "level_1_2":
                        xml_str += "<levelType>level_1_2</levelType>"
                        cmd_session += " level-1-2"
                if self.avoid_learning:
                    xml_str += "<avoidLearning>true</avoidLearning>"
                    cmd_session += " avoid-learning"
                elif not self.avoid_learning:
                    xml_str += "<avoidLearning>false</avoidLearning>"
                cmd_list.insert(0, cmd_session)
                self.updates_cmd.extend(cmd_list)

            if self.protocol:
                cmd_session = "import-route"
                if self.protocol == "rip":
                    xml_str = "<protocol>rip</protocol>"
                    cmd_session += " rip"
                elif self.protocol == "isis":
                    xml_str = "<protocol>isis</protocol>"
                    cmd_session += " isis"
                elif self.protocol == "ospf":
                    xml_str = "<protocol>ospf</protocol>"
                    cmd_session += " ospf"
                elif self.protocol == "static":
                    xml_str = "<protocol>static</protocol>"
                    cmd_session += " static"
                elif self.protocol == "direct":
                    xml_str = "<protocol>direct</protocol>"
                    cmd_session += " direct"
                elif self.protocol == "bgp":
                    xml_str = "<protocol>bgp</protocol>"
                    cmd_session += " bgp"
                    if self.permitibgp:
                        xml_str += "<permitIbgp>true</permitIbgp>"
                        cmd_session += " permit-ibgp"
                if self.protocol == "rip" or self.protocol == "isis" or self.protocol == "ospf":
                    xml_str += "<processId>%s</processId>" % self.processid
                    cmd_session += " %s" % self.processid
                if self.inheritcost:
                    xml_str += "<inheritCost>%s</inheritCost>" % self.inheritcost
                    cmd_session += " inherit-cost"
                if self.cost_type:
                    if self.cost_type == "external":
                        xml_str += "<costType>external</costType>"
                        cmd_session += " cost-type external"
                    elif self.cost_type == "internal":
                        xml_str += "<costType>internal</costType>"
                        cmd_session += " cost-type internal"
                if self.import_cost:
                    xml_str += "<cost>%s</cost>" % self.import_cost
                    cmd_session += " cost %s" % self.import_cost
                if self.import_tag:
                    xml_str += "<tag>%s</tag>" % self.import_tag
                    cmd_session += " tag %s" % self.import_tag
                if self.import_route_policy:
                    xml_str += "<policyType>routePolicy</policyType>"
                    xml_str += "<routePolicyName>%s</routePolicyName>" % self.import_route_policy
                    cmd_session += " route-policy %s" % self.import_route_policy
                if self.impotr_leveltype:
                    if self.impotr_leveltype == "level_1":
                        cmd_session += " level-1"
                    elif self.impotr_leveltype == "level_2":
                        cmd_session += " level-2"
                    elif self.impotr_leveltype == "level_1_2":
                        cmd_session += " level-1-2"
                cmd_list.insert(0, cmd_session)
                self.updates_cmd.extend(cmd_list)

            if self.bfd_min_rx or self.bfd_min_tx or self.bfd_multiplier_num:
                xml_str = ""
                self.updates_cmd.append("bfd all-interfaces enable")
                cmd_session = "bfd all-interfaces"
                if self.bfd_min_rx:
                    xml_str += "<bfdMinRx>%s</bfdMinRx>" % self.bfd_min_rx
                    cmd_session += " min-rx-interval %s" % self.bfd_min_rx
                if self.bfd_min_tx:
                    xml_str += "<bfdMinTx>%s</bfdMinTx>" % self.bfd_min_tx
                    cmd_session += " min-tx-interval %s" % self.bfd_min_tx
                if self.bfd_multiplier_num:
                    xml_str += "<bfdMultNum>%s</bfdMultNum>" % self.bfd_multiplier_num
                    cmd_session += " detect-multiplier %s" % self.bfd_multiplier_num
                cmd_list.insert(0, cmd_session)
                self.updates_cmd.extend(cmd_list)

            if self.export_protocol:
                cmd_session = "filter-policy"
                if self.export_aclnumorname:
                    xml_str = "<policyType>aclNumOrName</policyType>"
                    xml_str += "<aclNumOrName>%s</aclNumOrName>" % self.export_aclnumorname
                    if isinstance(self.export_aclnumorname, int):
                        cmd_session += " %s" % self.export_aclnumorname
                    elif isinstance(self.export_aclnumorname, str):
                        cmd_session += " acl-name %s" % self.export_aclnumorname
                if self.export_ipprefix:
                    xml_str = "<policyType>ipPrefix</policyType>"
                    xml_str += "<ipPrefix>%s</ipPrefix>" % self.export_ipprefix
                    cmd_session += " ip-prefix %s" % self.export_ipprefix
                if self.export_routepolicyname:
                    xml_str = "<policyType>routePolicy</policyType>"
                    xml_str += "<routePolicyName>%s</routePolicyName>" % self.export_routepolicyname
                    cmd_session += " route-policy %s" % self.export_routepolicyname
                xml_str += "<protocol>%s</protocol>" % self.export_protocol
                cmd_session += " export %s" % self.export_protocol
                if self.export_processid is not None:
                    xml_str += "<processId>%s</processId>" % self.export_processid
                    cmd_session += " %s" % self.export_processid
                cmd_list.insert(0, cmd_session)
                self.updates_cmd.extend(cmd_list)

            if self.import_ipprefix or self.import_aclnumorname or self.import_routepolicyname:
                cmd_session = "filter-policy"
                if self.import_aclnumorname:
                    xml_str = "<policyType>aclNumOrName</policyType>"
                    xml_str += "<aclNumOrName>%s</aclNumOrName>" % self.import_aclnumorname
                    if isinstance(self.import_aclnumorname, int):
                        cmd_session += " %s" % self.import_aclnumorname
                    elif isinstance(self.import_aclnumorname, str):
                        cmd_session += " acl-name %s" % self.import_aclnumorname
                if self.import_ipprefix:
                    xml_str = "<policyType>ipPrefix</policyType>"
                    xml_str += "<ipPrefix>%s</ipPrefix>" % self.import_ipprefix
                    cmd_session += " ip-prefix %s" % self.import_ipprefix
                if self.import_routepolicyname:
                    xml_str = "<policyType>routePolicy</policyType>"
                    xml_str += "<routePolicyName>%s</routePolicyName>" % self.import_routepolicyname
                    cmd_session += " route-policy %s" % self.import_routepolicyname
                cmd_session += "import"
                cmd_list.insert(0, cmd_session)
                self.updates_cmd.extend(cmd_list)
        else:
            # absent
            if self.description and self.description == instance.get("description"):
                xml_str += "<description>%s</description>" % self.description
                self.updates_cmd.append("undo description")

            if self.islevel and self.islevel == instance.get("isLevel"):
                xml_str += "<isLevel>level_1_2</isLevel>"
                self.updates_cmd.append("undo is-level")

            if self.coststyle and self.coststyle == instance.get("costStyle"):
                xml_str += "<costStyle>%s</costStyle>" % ("narrow")
                xml_str += "<relaxSpfLimit>false</relaxSpfLimit>"
                self.updates_cmd.append("undo cost-style")

            if self.stdlevel1cost and str(self.stdlevel1cost) == instance.get("stdLevel1Cost"):
                xml_str += "<stdLevel1Cost>%s</stdLevel1Cost>" % self.stdlevel1cost
                self.updates_cmd.append("undo circuit-cost %s level-1" % self.stdlevel1cost)

            if self.stdlevel2cost and str(self.stdlevel2cost) == instance.get("stdLevel2Cost"):
                xml_str += "<stdLevel2Cost>%s</stdLevel2Cost>" % self.stdlevel2cost
                self.updates_cmd.append("undo circuit-cost %s level-2" % self.stdlevel2cost)

            if self.stdbandwidth and str(self.stdbandwidth) == instance.get("stdbandwidth"):
                xml_str += "<stdbandwidth>100</stdbandwidth>"
                self.updates_cmd.append("undo bandwidth-reference")

            if self.netentity and self.netentity == instance.get("netEntity"):
                xml_str = CE_NC_DELATE_ISIS_ENTITY % (self.instance_id, self.netentity)
                self.updates_cmd.append("undo network-entity %s" % self.netentity)

            if self.preference_value or self.route_policy_name:
                xml_str = ""
                if self.preference_value and str(self.preference_value) == instance.get("preferenceValue"):
                    xml_str = "<preferenceValue>%s</preferenceValue>" % self.preference_value
                    if self.route_policy_name and self.route_policy_name == instance.get("routePolicyName"):
                        xml_str += "<routePolicyName>%s</routePolicyName>" % self.route_policy_name
                    self.updates_cmd.append("undo preference")
                elif not self.preference_value and self.route_policy_name and self.route_policy_name == instance.get("routePolicyName"):
                    xml_str = "<routePolicyName>%s</routePolicyName>" % self.route_policy_name
                    self.updates_cmd.append("undo preference")
                xml_str = CE_NC_DELETE_ISIS_PREFERENCE % (self.instance_id, xml_str)

            if self.max_load and str(self.max_load) == instance.get("maxLoadBalancing"):
                xml_str = CE_NC_DELETE_ISIS_MAXLOAD % self.instance_id
                self.updates_cmd.append("undo maximum load-balancing")

            if self.ip_address:
                xml_str = CE_NC_DELETE_ISIS_NEXTHOP % (self.instance_id, self.ip_address)
                self.updates_cmd.append("undo nexthop %s" % self.ip_address)

            if self.penetration_direct:
                if self.penetration_direct == "level2-level1":
                    self.updates_cmd.append("undo import-route isis level-2 into level-1")
                elif self.penetration_direct == "level1-level2":
                    self.updates_cmd.append("undo import-route isis level-1 into level-2")
                    self.updates_cmd.append("import-route isis level-1 into level-2 disable")

            if self.bfd_min_rx or self.bfd_min_tx or self.bfd_multiplier_num is not None:
                xml_str = CE_NC_DELETE_ISIS_BFDLINK % self.instance_id
                self.updates_cmd.append("undo bfd all-interfaces enable")
                cmd_session = "undo bfd all-interfaces"
                if self.bfd_min_rx:
                    cmd_session += " min-rx-interval %s" % self.bfd_min_rx
                if self.bfd_min_tx:
                    cmd_session += " min-tx-interval %s" % self.bfd_min_tx
                if self.bfd_multiplier_num:
                    cmd_session += " detect-multiplier %s" % self.bfd_multiplier_num
                cmd_list.insert(0, cmd_session)
                self.updates_cmd.extend(cmd_list)

            if self.defaultmode:
                xml_str = CE_NC_DELETE_ISIS_DEFAULTROUTE % self.instance_id
                self.updates_cmd.append("undo default-route-advertise")

            if self.protocol:
                if self.protocol == "rip" or self.protocol == "isis" or self.protocol == "ospf":
                    self.updates_cmd.append("undo import-route %s %s" % (self.protocol, self.processid))
                else:
                    self.updates_cmd.append("undo import-route %s" % self.protocol)

            if self.export_protocol:
                cmd_session = "undo filter-policy"
                if self.export_aclnumorname:
                    if isinstance(self.export_aclnumorname, int):
                        cmd_session += " %s" % self.export_aclnumorname
                    elif isinstance(self.export_aclnumorname, str):
                        cmd_session += " acl-name %s" % self.export_aclnumorname
                if self.export_ipprefix:
                    cmd_session += " ip-prefix %s" % self.export_ipprefix
                if self.export_routepolicyname:
                    cmd_session += " route-policy %s" % self.export_routepolicyname
                cmd_session += " export %s" % self.export_protocol
                if self.export_processid is not None:
                    cmd_session += " %s" % self.export_processid
                cmd_list.insert(0, cmd_session)
                self.updates_cmd.extend(cmd_list)
            if self.import_ipprefix or self.import_aclnumorname or self.import_routepolicyname:
                cmd_session = "undo filter-policy"
                if self.import_aclnumorname:
                    if isinstance(self.import_aclnumorname, int):
                        cmd_session += " %s" % self.import_aclnumorname
                    elif isinstance(self.import_aclnumorname, str):
                        cmd_session += " acl-name %s" % self.import_aclnumorname
                if self.import_ipprefix:
                    cmd_session += " ip-prefix %s" % self.import_ipprefix
                if self.import_routepolicyname:
                    cmd_session += " route-policy %s" % self.import_routepolicyname
                cmd_session += " import"
                cmd_list.insert(0, cmd_session)
                self.updates_cmd.extend(cmd_list)

        if self.autocostenable and instance.get("stdAutoCostEnable", "false") == "false":
            xml_str += "<stdAutoCostEnable>true</stdAutoCostEnable>"
            self.updates_cmd.append("auto-cost enable")
        elif not self.autocostenable and instance.get("stdAutoCostEnable", "false") == "true":
            xml_str += "<stdAutoCostEnable>false</stdAutoCostEnable>"
            xml_str += "<stdAutoCostEnableCompatible>false</stdAutoCostEnableCompatible>"
            self.updates_cmd.append("undo auto-cost enable")

        if self.autocostenable:
            if self.autocostenablecompatible and instance.get("stdAutoCostEnableCompatible", "false") == "false":
                xml_str += "<stdAutoCostEnableCompatible>true</stdAutoCostEnableCompatible>"
                self.updates_cmd.append("auto-cost enable compatible")
            elif not self.autocostenablecompatible and instance.get("stdAutoCostEnableCompatible", "false") == "true":
                xml_str += "<stdAutoCostEnableCompatible>false</stdAutoCostEnableCompatible>"
                self.updates_cmd.append("auto-cost enable")

        if self.state == "present":
            if self.netentity or self.preference_value or self.route_policy_name or self.max_load or self.ip_address:
                return xml_str
            elif self.penetration_direct:
                if self.penetration_direct == "level2-level1":
                    return CE_NC_MERGE_ISIS_LEAKROUTELEVEL2 % (self.instance_id, xml_str)
                elif self.penetration_direct == "level1-level2":
                    return CE_NC_MERGE_ISIS_LEAKROUTELEVEL1 % (self.instance_id, xml_str)
            elif self.defaultmode:
                return CE_NC_MERGE_ISIS_DEFAULTROUTE % (self.instance_id, xml_str)
            elif self.protocol:
                return CE_NC_MERGE_ISIS_IMPORTROUTE % (self.instance_id, xml_str)
            elif self.export_protocol:
                return CE_NC_MERGE_ISIS_EXPORTROUTE % (self.instance_id, xml_str)
            elif self.import_routepolicyname or self.import_aclnumorname or self.import_ipprefix:
                return CE_NC_MERGE_ISIS_IMPORTIPROUTE % (self.instance_id, xml_str)
            elif self.bfd_min_rx or self.bfd_min_tx or self.bfd_multiplier_num:
                return CE_NC_MERGE_ISIS_BFDLINK % (self.instance_id, xml_str)
            else:
                return '<isSites><isSite operation="merge">' + xml_str + '</isSite></isSites>'
        else:
            if self.netentity or self.preference_value or self.route_policy_name or self.max_load \
                    or self.ip_address or self.defaultmode or self.bfd_min_rx or self.bfd_min_tx or self.bfd_multiplier_num is not None:
                return xml_str
            else:
                return '<isSites><isSite operation="delete">' + xml_str + '</isSite></isSites>'

    def netconf_load_config(self, xml_str):
        """load bfd config by netconf"""

        if not xml_str:
            return
        if xml_str == "<instanceId>%s</instanceId>" % self.instance_id:
            pass
        else:
            xml_cfg = """
                <config>
                <isiscomm xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
                %s
                </isiscomm>
                </config>""" % xml_str
            set_nc_config(self.module, xml_cfg)
            self.changed = True

    def check_params(self):
        """Check all input params"""
        levelcost = 16777215
        if not self.instance_id:
            self.module.fail_json(msg="Error: Missing required arguments: instance_id.")

        if self.instance_id:
            if self.instance_id < 1 or self.instance_id > 4294967295:
                self.module.fail_json(msg="Error: Instance id is not ranges from 1 to 4294967295.")

        # check description
        if self.description:
            if len(self.description) < 1 or len(self.description) > 80:
                self.module.fail_json(msg="Error: description is invalid.")

        #
        if self.stdbandwidth:
            if self.stdbandwidth < 1 or self.stdbandwidth > 2147483648:
                self.module.fail_json(msg="Error: stdbandwidth is not ranges from 1 to 2147483648.")

        if self.relaxspfLimit is not None and not self.coststyle:
            self.module.fail_json(msg="Error: relaxspfLimit must set after coststyle.")

        if self.coststyle:
            if self.coststyle != "wide" and self.coststyle != "wtransition":
                levelcost = 63
            else:
                levelcost = 16777215
        if self.stdlevel1cost:
            if self.stdlevel1cost < 1 or self.stdlevel1cost > levelcost:
                self.module.fail_json(msg="Error: stdlevel1cost is not ranges from 1 to %s." % levelcost)

        if self.stdlevel2cost:
            if self.stdlevel2cost < 1 or self.stdlevel2cost > levelcost:
                self.module.fail_json(msg="Error: stdlevel2cost is not ranges from 1 to %s." % levelcost)

        if self.coststyle:
            if self.coststyle != "ntransition" and self.coststyle != "transition":
                if self.relaxspfLimit:
                    self.module.fail_json(msg="Error: relaxspfLimit can not be set while the coststyle is not ntransition or transition")

        if self.autocostenablecompatible:
            if not self.autocostenable:
                self.module.fail_json(msg="Error: you shoule enable the autocostenable first.")

        if self.preference_value:
            if self.preference_value < 1 or self.preference_value > 255:
                self.module.fail_json(msg="Error: preference_value is not ranges from 1 to 255.")

        if self.route_policy_name:
            if len(self.route_policy_name) < 1 or len(self.route_policy_name) > 200:
                self.module.fail_json(msg="Error: route_policy_name is invalid.")

        if self.max_load:
            if self.max_load < 1 or self.max_load > 32:
                self.module.fail_json(msg="Error: max_load is not ranges from 1 to 32.")

        if self.weight:
            if self.weight < 1 or self.weight > 254:
                self.module.fail_json(msg="Error: weight is not ranges from 1 to 254.")

        if self.aclnum_or_name:
            if isinstance(self.aclnum_or_name, int):
                if self.aclnum_or_name < 2000 or self.aclnum_or_name > 2999:
                    self.module.fail_json(msg="Error: acl_num is not ranges from 2000 to 2999.")
            elif isinstance(self.aclnum_or_name, str):
                if len(self.aclnum_or_name) < 1 or len(self.aclnum_or_name) > 32:
                    self.module.fail_json(msg="Error: acl_name is invalid.")
        if self.ip_prefix_name:
            if len(self.ip_prefix_name) < 1 or len(self.ip_prefix_name) > 169:
                self.module.fail_json(msg="Error: ip_prefix_name is invalid.")
        if self.import_routepolicy_name:
            if len(self.import_routepolicy_name) < 1 or len(self.import_routepolicy_name) > 200:
                self.module.fail_json(msg="Error: import_routepolicy_name is invalid.")
        if self.tag:
            if self.tag < 1 or self.tag > 4294967295:
                self.module.fail_json(msg="Error: tag is not ranges from 1 to 4294967295.")

        if self.mode_routepolicyname:
            if len(self.mode_routepolicyname) < 1 or len(self.mode_routepolicyname) > 200:
                self.module.fail_json(msg="Error: mode_routepolicyname is invalid.")
        if self.cost is not None:
            if self.cost < 0 or self.cost > 4261412864:
                self.module.fail_json(msg="Error: cost is not ranges from 0 to 4261412864.")
        if self.mode_tag:
            if self.mode_tag < 1 or self.mode_tag > 4294967295:
                self.module.fail_json(msg="Error: mode_tag is not ranges from 1 to 4294967295.")

        if self.processid is not None:
            if self.processid < 0 or self.processid > 4294967295:
                self.module.fail_json(msg="Error: processid is not ranges from 0 to 4294967295.")

        if self.import_cost is not None:
            if self.import_cost < 0 or self.import_cost > 4261412864:
                self.module.fail_json(msg="Error: import_cost is not ranges from 0 to 4261412864.")

        if self.import_tag:
            if self.import_tag < 1 or self.import_tag > 4294967295:
                self.module.fail_json(msg="Error: import_tag is not ranges from 1 to 4294967295.")

        if self.export_aclnumorname:
            if isinstance(self.export_aclnumorname, int):
                if self.export_aclnumorname < 2000 or self.export_aclnumorname > 2999:
                    self.module.fail_json(msg="Error: acl_num is not ranges from 2000 to 2999.")
            elif isinstance(self.export_aclnumorname, str):
                if len(self.export_aclnumorname) < 1 or len(self.export_aclnumorname) > 32:
                    self.module.fail_json(msg="Error: acl_name is invalid.")

        if self.export_processid:
            if self.export_processid < 1 or self.export_processid > 4294967295:
                self.module.fail_json(msg="Error: export_processid is not ranges from 1 to 4294967295.")

        if self.export_ipprefix:
            if len(self.export_ipprefix) < 1 or len(self.export_ipprefix) > 169:
                self.module.fail_json(msg="Error: export_ipprefix is invalid.")

        if self.export_routepolicyname:
            if len(self.export_routepolicyname) < 1 or len(self.export_routepolicyname) > 200:
                self.module.fail_json(msg="Error: export_routepolicyname is invalid.")

        if self.bfd_min_rx:
            if self.bfd_min_rx < 50 or self.bfd_min_rx > 1000:
                self.module.fail_json(msg="Error: bfd_min_rx is not ranges from 50 to 1000.")

        if self.bfd_min_tx:
            if self.bfd_min_tx < 50 or self.bfd_min_tx > 1000:
                self.module.fail_json(msg="Error: bfd_min_tx is not ranges from 50 to 1000.")

        if self.bfd_multiplier_num:
            if self.bfd_multiplier_num < 3 or self.bfd_multiplier_num > 50:
                self.module.fail_json(msg="Error: bfd_multiplier_num is not ranges from 3 to 50.")

        if self.import_routepolicyname:
            if len(self.import_routepolicyname) < 1 or len(self.import_routepolicyname) > 200:
                self.module.fail_json(msg="Error: import_routepolicyname is invalid.")

        if self.import_aclnumorname:
            if isinstance(self.import_aclnumorname, int):
                if self.import_aclnumorname < 2000 or self.import_aclnumorname > 2999:
                    self.module.fail_json(msg="Error: acl_num is not ranges from 2000 to 2999.")
            elif isinstance(self.import_aclnumorname, str):
                if len(self.import_aclnumorname) < 1 or len(self.import_aclnumorname) > 32:
                    self.module.fail_json(msg="Error: acl_name is invalid.")

    def get_proposed(self):
        """get proposed info"""
        # base config
        self.proposed["instance_id"] = self.instance_id
        self.proposed["description"] = self.description
        self.proposed["islevel"] = self.islevel
        self.proposed["coststyle"] = self.coststyle
        self.proposed["relaxspfLimit"] = self.relaxspfLimit
        self.proposed["stdlevel1cost"] = self.stdlevel1cost
        self.proposed["stdlevel2cost"] = self.stdlevel2cost
        self.proposed["stdbandwidth"] = self.stdbandwidth
        self.proposed["autocostenable"] = self.autocostenable
        self.proposed["autocostenablecompatible"] = self.autocostenablecompatible
        self.proposed["netentity"] = self.netentity
        self.proposed["preference_value"] = self.preference_value
        self.proposed["route_policy_name"] = self.route_policy_name
        self.proposed["max_load"] = self.max_load
        self.proposed["ip_address"] = self.ip_address
        self.proposed["weight"] = self.weight
        self.proposed["penetration_direct"] = self.penetration_direct
        self.proposed["aclnum_or_name"] = self.aclnum_or_name
        self.proposed["ip_prefix_name"] = self.ip_prefix_name
        self.proposed["import_routepolicy_name"] = self.import_routepolicy_name
        self.proposed["tag"] = self.tag
        self.proposed["allow_filter"] = self.allow_filter
        self.proposed["allow_up_down"] = self.allow_up_down
        self.proposed["enablelevel1tolevel2"] = self.enablelevel1tolevel2
        self.proposed["protocol"] = self.protocol
        self.proposed["processid"] = self.processid
        self.proposed["cost_type"] = self.cost_type
        self.proposed["import_cost"] = self.import_cost
        self.proposed["import_tag"] = self.import_tag
        self.proposed["import_route_policy"] = self.import_route_policy
        self.proposed["impotr_leveltype"] = self.impotr_leveltype
        self.proposed["inheritcost"] = self.inheritcost
        self.proposed["permitibgp"] = self.permitibgp
        self.proposed["export_protocol"] = self.export_protocol
        self.proposed["export_policytype"] = self.export_policytype
        self.proposed["export_processid"] = self.export_processid
        self.proposed["export_aclnumorname"] = self.export_aclnumorname
        self.proposed["export_ipprefix"] = self.export_ipprefix
        self.proposed["export_routepolicyname"] = self.export_routepolicyname
        self.proposed["import_aclnumorname"] = self.import_aclnumorname
        self.proposed["import_ipprefix"] = self.import_ipprefix
        self.proposed["import_routepolicyname"] = self.import_routepolicyname
        self.proposed["bfd_min_rx"] = self.bfd_min_rx
        self.proposed["bfd_min_tx"] = self.bfd_min_tx
        self.proposed["bfd_multiplier_num"] = self.bfd_multiplier_num
        self.proposed["state"] = self.state

    def get_existing(self):
        """get existing info"""

        if not self.isis_dict:
            self.existing["instance"] = None
        else:
            self.existing["instance"] = self.isis_dict.get("instance")

    def get_end_state(self):
        """get end state info"""

        isis_dict = self.get_isis_dict()
        if not isis_dict:
            self.end_state["instance"] = None
        else:
            self.end_state["instance"] = isis_dict.get("instance")
        if self.end_state == self.existing:
            self.changed = False

    def work(self):
        """worker"""

        self.check_params()
        self.isis_dict = self.get_isis_dict()
        self.get_existing()
        self.get_proposed()

        # deal present or absent
        xml_str = ''
        if self.instance_id:
            xml_str += self.config_session()
        # update to device
        if xml_str:
            self.netconf_load_config(xml_str)
            self.changed = True

        self.get_end_state()
        self.results['changed'] = self.changed
        self.results['proposed'] = self.proposed
        self.results['existing'] = self.existing
        self.results['end_state'] = self.end_state
        if self.changed:
            self.results['updates'] = self.updates_cmd
        else:
            self.results['updates'] = list()
        self.module.exit_json(**self.results)


def main():
    """Module main"""

    argument_spec = dict(
        instance_id=dict(required=True, type='int'),
        description=dict(required=False, type='str'),
        islevel=dict(required=False, type='str', choices=['level_1', 'level_2', 'level_1_2']),
        coststyle=dict(required=False, type='str', choices=['narrow', 'wide', 'transition', 'ntransition', 'wtransition']),
        relaxspfLimit=dict(required=False, type='bool'),
        stdlevel1cost=dict(required=False, type='int'),
        stdlevel2cost=dict(required=False, type='int'),
        stdbandwidth=dict(required=False, type='int'),
        autocostenable=dict(required=False, type='bool'),
        autocostenablecompatible=dict(required=False, type='bool'),
        netentity=dict(required=False, type='str'),
        preference_value=dict(required=False, type='int'),
        route_policy_name=dict(required=False, type='str'),
        max_load=dict(required=False, type='int'),
        ip_address=dict(required=False, type='str'),
        weight=dict(required=False, type='int'),
        penetration_direct=dict(required=False, type='str', choices=['level2-level1', 'level1-level2']),
        aclnum_or_name=dict(required=False, type='str'),
        ip_prefix_name=dict(required=False, type='str'),
        import_routepolicy_name=dict(required=False, type='str'),
        tag=dict(required=False, type='int'),
        allow_filter=dict(required=False, type='bool'),
        allow_up_down=dict(required=False, type='bool'),
        enablelevel1tolevel2=dict(required=False, type='bool'),
        defaultmode=dict(required=False, type='str', choices=['always', 'matchDefault', 'matchAny']),
        mode_routepolicyname=dict(required=False, type='str'),
        cost=dict(required=False, type='int'),
        mode_tag=dict(required=False, type='int'),
        level_type=dict(required=False, type='str', choices=['level_1', 'level_2', 'level_1_2']),
        avoid_learning=dict(required=False, type='bool'),
        protocol=dict(required=False, type='str', choices=['direct', 'ospf', 'isis', 'static', 'rip', 'bgp', 'ospfv3', 'all']),
        processid=dict(required=False, type='int'),
        cost_type=dict(required=False, type='str', choices=['external', 'internal']),
        import_cost=dict(required=False, type='int'),
        import_tag=dict(required=False, type='int'),
        import_route_policy=dict(required=False, type='str'),
        impotr_leveltype=dict(required=False, type='str', choices=['level_1', 'level_2', 'level_1_2']),
        inheritcost=dict(required=False, type='bool'),
        permitibgp=dict(required=False, type='bool'),
        export_protocol=dict(required=False, type='str', choices=['direct', 'ospf', 'isis', 'static', 'rip', 'bgp', 'ospfv3', 'all']),
        export_policytype=dict(required=False, type='str', choices=['aclNumOrName', 'ipPrefix', 'routePolicy']),
        export_processid=dict(required=False, type='int'),
        export_aclnumorname=dict(required=False, type='str'),
        export_ipprefix=dict(required=False, type='str'),
        export_routepolicyname=dict(required=False, type='str'),
        import_aclnumorname=dict(required=False, type='str'),
        import_ipprefix=dict(required=False, type='str'),
        import_routepolicyname=dict(required=False, type='str'),
        bfd_min_rx=dict(required=False, type='int'),
        bfd_min_tx=dict(required=False, type='int'),
        bfd_multiplier_num=dict(required=False, type='int'),
        state=dict(required=False, default='present', choices=['present', 'absent'])
    )

    module = ISIS_View(argument_spec)
    module.work()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2019, Kevin Breit (@kbreit) <kevin.breit@kevinbreit.net>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {
    'metadata_version': '1.1',
    'status': ['preview'],
    'supported_by': 'community'
}

DOCUMENTATION = r'''
---
module: meraki_webhook
short_description: Manage webhooks configured in the Meraki cloud
description:
- Configure and query information about webhooks within the Meraki cloud.
notes:
- Some of the options are likely only used for developers within Meraki.
options:
    state:
      description:
      - Specifies whether object should be queried, created/modified, or removed.
      choices: [absent, present, query]
      default: query
      type: str
    net_name:
      description:
      - Name of network which configuration is applied to.
      aliases: [network]
      type: str
    net_id:
      description:
      - ID of network which configuration is applied to.
      type: str
    name:
      description:
      - Name of webhook.
      type: str
    shared_secret:
      description:
      - Secret password to use when accessing webhook.
      type: str
    url:
      description:
      - URL to access when calling webhook.
      type: str
    webhook_id:
      description:
      - Unique ID of webhook.
      type: str
    test:
      description:
      - Indicates whether to test or query status.
      type: str
      choices: [test]
    test_id:
      description:
      - ID of webhook test query.
      type: str
author:
- Kevin Breit (@kbreit)
extends_documentation_fragment: cisco.meraki.meraki
'''

EXAMPLES = r'''
- name: Create webhook
  meraki_webhook:
    auth_key: abc123
    state: present
    org_name: YourOrg
    net_name: YourNet
    name: Test_Hook
    url: https://webhook.url/
    shared_secret: shhhdonttellanyone
  delegate_to: localhost

- name: Query one webhook
  meraki_webhook:
    auth_key: abc123
    state: query
    org_name: YourOrg
    net_name: YourNet
    name: Test_Hook
  delegate_to: localhost

- name: Query all webhooks
  meraki_webhook:
    auth_key: abc123
    state: query
    org_name: YourOrg
    net_name: YourNet
  delegate_to: localhost

- name: Delete webhook
  meraki_webhook:
    auth_key: abc123
    state: absent
    org_name: YourOrg
    net_name: YourNet
    name: Test_Hook
  delegate_to: localhost

- name: Test webhook
  meraki_webhook:
    auth_key: abc123
    state: present
    org_name: YourOrg
    net_name: YourNet
    test: test
    url: https://webhook.url/abc123
  delegate_to: localhost

- name: Get webhook status
  meraki_webhook:
    auth_key: abc123
    state: present
    org_name: YourOrg
    net_name: YourNet
    test: status
    test_id: abc123531234
  delegate_to: localhost
'''

RETURN = r'''
data:
    description: List of administrators.
    returned: success
    type: complex
    contains:
        id:
          description: Unique ID of webhook.
          returned: success
          type: str
          sample: aHR0cHM6Ly93ZWJob22LnvpdGUvOGViNWI3NmYtYjE2Ny00Y2I4LTlmYzQtND32Mj3F5NzIaMjQ0
        name:
          description: Descriptive name of webhook.
          returned: success
          type: str
          sample: Test_Hook
        networkId:
          description: ID of network containing webhook object.
          returned: success
          type: str
          sample: N_12345
        shared_secret:
          description: Password for webhook.
          returned: success
          type: str
          sample: VALUE_SPECIFIED_IN_NO_LOG_PARAMETER
        url:
          description: URL of webhook endpoint.
          returned: success
          type: str
          sample: https://webhook.url/abc123
        status:
          description: Status of webhook test.
          returned: success, when testing webhook
          type: str
          sample: enqueued
'''

from ansible.module_utils.basic import AnsibleModule, json
from ansible_collections.cisco.meraki.plugins.module_utils.network.meraki.meraki import MerakiModule, meraki_argument_spec


def get_webhook_id(name, webhooks):
    for webhook in webhooks:
        if name == webhook['name']:
            return webhook['id']
    return None


def get_all_webhooks(meraki, net_id):
    path = meraki.construct_path('get_all', net_id=net_id)
    response = meraki.request(path, method='GET')
    if meraki.status == 200:
        return response


def sanitize_no_log_values(meraki):
    try:
        meraki.result['diff']['before']['shared_secret'] = 'VALUE_SPECIFIED_IN_NO_LOG_PARAMETER'
    except KeyError:
        pass
    try:
        meraki.result['data'][0]['shared_secret'] = 'VALUE_SPECIFIED_IN_NO_LOG_PARAMETER'
    except KeyError:
        pass
    try:
        meraki.result['data']['shared_secret'] = 'VALUE_SPECIFIED_IN_NO_LOG_PARAMETER'
    except (KeyError, TypeError) as e:
        pass


def main():
    # define the available arguments/parameters that a user can pass to
    # the module

    argument_spec = meraki_argument_spec()
    argument_spec.update(state=dict(type='str', choices=['absent', 'present', 'query'], default='query'),
                         net_name=dict(type='str', aliases=['network']),
                         net_id=dict(type='str'),
                         name=dict(type='str'),
                         url=dict(type='str'),
                         shared_secret=dict(type='str', no_log=True),
                         webhook_id=dict(type='str'),
                         test=dict(type='str', choices=['test']),
                         test_id=dict(type='str'),
                         )

    # the AnsibleModule object will be our abstraction working with Ansible
    # this includes instantiation, a couple of common attr would be the
    # args/params passed to the execution, as well as if the module
    # supports check mode
    module = AnsibleModule(argument_spec=argument_spec,
                           supports_check_mode=True,
                           )
    meraki = MerakiModule(module, function='webhooks')

    meraki.params['follow_redirects'] = 'all'

    query_url = {'webhooks': '/networks/{net_id}/webhooks/httpServers'}
    query_one_url = {'webhooks': '/networks/{net_id}/webhooks/httpServers/{hookid}'}
    create_url = {'webhooks': '/networks/{net_id}/webhooks/httpServers'}
    update_url = {'webhooks': '/networks/{net_id}/webhooks/httpServers/{hookid}'}
    delete_url = {'webhooks': '/networks/{net_id}/webhooks/httpServers/{hookid}'}
    test_url = {'webhooks': '/networks/{net_id}/webhooks/webhookTests'}
    test_status_url = {'webhooks': '/networks/{net_id}/webhooks/webhookTests/{testid}'}

    meraki.url_catalog['get_all'].update(query_url)
    meraki.url_catalog['get_one'].update(query_one_url)
    meraki.url_catalog['create'] = create_url
    meraki.url_catalog['update'] = update_url
    meraki.url_catalog['delete'] = delete_url
    meraki.url_catalog['test'] = test_url
    meraki.url_catalog['test_status'] = test_status_url

    org_id = meraki.params['org_id']
    if org_id is None:
        org_id = meraki.get_org_id(meraki.params['org_name'])
    net_id = meraki.params['net_id']
    if net_id is None:
        nets = meraki.get_nets(org_id=org_id)
        net_id = meraki.get_net_id(net_name=meraki.params['net_name'], data=nets)
    webhook_id = meraki.params['webhook_id']
    webhooks = None
    if webhook_id is None and meraki.params['name']:
        webhooks = get_all_webhooks(meraki, net_id)
        webhook_id = get_webhook_id(meraki.params['name'], webhooks)

    if meraki.params['state'] == 'present' and meraki.params['test'] is None:
        payload = {'name': meraki.params['name'],
                   'url': meraki.params['url'],
                   'sharedSecret': meraki.params['shared_secret']}

    if meraki.params['state'] == 'query':
        if webhook_id is not None:  # Query a single webhook
            path = meraki.construct_path('get_one', net_id=net_id, custom={'hookid': webhook_id})
            response = meraki.request(path, method='GET')
            if meraki.status == 200:
                meraki.result['data'] = response
                sanitize_no_log_values(meraki)
                meraki.exit_json(**meraki.result)
        elif meraki.params['test_id'] is not None:
            path = meraki.construct_path('test_status', net_id=net_id, custom={'testid': meraki.params['test_id']})
            response = meraki.request(path, method='GET')
            if meraki.status == 200:
                meraki.result['data'] = response
                sanitize_no_log_values(meraki)
                meraki.exit_json(**meraki.result)
        else:
            path = meraki.construct_path('get_all', net_id=net_id)
            response = meraki.request(path, method='GET')
            if meraki.status == 200:
                meraki.result['data'] = response
                # meraki.fail_json(msg=meraki.result)
                sanitize_no_log_values(meraki)
                meraki.exit_json(**meraki.result)
    elif meraki.params['state'] == 'present':
        if meraki.params['test'] == 'test':
            payload = {'url': meraki.params['url']}
            path = meraki.construct_path('test', net_id=net_id)
            response = meraki.request(path, method='POST', payload=json.dumps(payload))
            if meraki.status == 201:
                meraki.result['data'] = response
                meraki.exit_json(**meraki.result)
        if webhook_id is None:  # New webhook needs to be created
            if meraki.check_mode is True:
                meraki.result['data'] = payload
                meraki.result['data']['networkId'] = net_id
                meraki.result['changed'] = True
                meraki.exit_json(**meraki.result)
            path = meraki.construct_path('create', net_id=net_id)
            response = meraki.request(path, method='POST', payload=json.dumps(payload))
            if meraki.status == 201:
                meraki.result['data'] = response
                meraki.result['changed'] = True
        else:  # Need to update
            path = meraki.construct_path('get_one', net_id=net_id, custom={'hookid': webhook_id})
            original = meraki.request(path, method='GET')
            if meraki.is_update_required(original, payload):
                if meraki.check_mode is True:
                    meraki.generate_diff(original, payload)
                    sanitize_no_log_values(meraki)
                    original.update(payload)
                    meraki.result['data'] = original
                    meraki.result['changed'] = True
                    meraki.exit_json(**meraki.result)
                path = meraki.construct_path('update', net_id=net_id, custom={'hookid': webhook_id})
                response = meraki.request(path, method='PUT', payload=json.dumps(payload))
                if meraki.status == 200:
                    meraki.generate_diff(original, response)
                    sanitize_no_log_values(meraki)
                    meraki.result['data'] = response
                    meraki.result['changed'] = True
            else:
                meraki.result['data'] = original
    elif meraki.params['state'] == 'absent':
        if webhook_id is None:  # Make sure it is downloaded
            if webhooks is None:
                webhooks = get_all_webhooks(meraki, net_id)
            webhook_id = get_webhook_id(meraki.params['name'], webhooks)
            if webhook_id is None:
                meraki.fail_json(msg="There is no webhook with the name {0}".format(meraki.params['name']))
        if webhook_id:  # Test to see if it exists
            if meraki.module.check_mode is True:
                meraki.result['data'] = None
                meraki.result['changed'] = True
                meraki.exit_json(**meraki.result)
            path = meraki.construct_path('delete', net_id=net_id, custom={'hookid': webhook_id})
            response = meraki.request(path, method='DELETE')
            if meraki.status == 204:
                meraki.result['data'] = response
                meraki.result['changed'] = True

    # in the event of a successful module execution, you will want to
    # simple AnsibleModule.exit_json(), passing the key/value results
    meraki.exit_json(**meraki.result)


if __name__ == '__main__':
    main()



from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import pytest

from awx.main.models import WorkflowJobTemplate, User


@pytest.mark.django_db
@pytest.mark.parametrize('state', ('present', 'absent'))
def test_grant_organization_permission(run_module, admin_user, organization, state):
    rando = User.objects.create(username='rando')
    if state == 'absent':
        organization.admin_role.members.add(rando)

    result = run_module('tower_role', {
        'user': rando.username,
        'organization': organization.name,
        'role': 'admin',
        'state': state
    }, admin_user)
    assert not result.get('failed', False), result.get('msg', result)

    if state == 'present':
        assert rando in organization.execute_role
    else:
        assert rando not in organization.execute_role


@pytest.mark.django_db
@pytest.mark.parametrize('state', ('present', 'absent'))
def test_grant_workflow_permission(run_module, admin_user, organization, state):
    wfjt = WorkflowJobTemplate.objects.create(organization=organization, name='foo-workflow')
    rando = User.objects.create(username='rando')
    if state == 'absent':
        wfjt.execute_role.members.add(rando)

    result = run_module('tower_role', {
        'user': rando.username,
        'workflow': wfjt.name,
        'role': 'execute',
        'state': state
    }, admin_user)
    assert not result.get('failed', False), result.get('msg', result)

    if state == 'present':
        assert rando in wfjt.execute_role
    else:
        assert rando not in wfjt.execute_role


@pytest.mark.django_db
def test_invalid_role(run_module, admin_user, project):
    rando = User.objects.create(username='rando')
    result = run_module('tower_role', {
        'user': rando.username,
        'project': project.name,
        'role': 'adhoc',
        'state': 'present'
    }, admin_user)
    assert result.get('failed', False)
    msg = result.get('msg')
    assert 'has no role adhoc_role' in msg
    assert 'available roles: admin_role, use_role, update_role, read_role' in msg



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

"""
The Ospfv3 parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.network_template import (
    NetworkTemplate,
)


def _tmplt_area_nssa(area):
    nssa = area["nssa"]
    command = "area {area_id} nssa".format(**area)
    if nssa.get("set") is False:
        command = "no {0}".format(command)
    else:
        for attrib in [
            "no_summary",
            "no_redistribution",
            "default_information_originate",
        ]:
            if nssa.get(attrib):
                command += " {0}".format(attrib.replace("_", "-"))
        if nssa.get("route_map"):
            command += " route-map {route_map}".format(**nssa)
    return command


def _tmplt_area_nssa_translate(area):
    translate = area["nssa"]["translate"]["type7"]
    command = "area {area_id} nssa translate type7".format(**area)
    for attrib in ["always", "never", "supress_fa"]:
        if translate.get(attrib):
            command += " {0}".format(attrib.replace("_", "-"))
    return command


def _tmplt_area_stub(area):
    stub = area["stub"]
    command = "area {area_id} stub".format(**area)
    if stub.get("set") is False:
        command = "no {0}".format(command)
    elif stub.get("no_summary"):
        command += " no-summary"
    return command


def _tmplt_log_adjacency_changes(proc):
    command = "log-adjacency-changes"
    if proc.get("log_adjacency_changes").get("detail", False) is True:
        command += " detail"
    return command


def _tmplt_max_lsa(proc):
    max_lsa = proc["max_lsa"]
    command = "max-lsa {max_non_self_generated_lsa}".format(**max_lsa)
    if max_lsa.get("threshold"):
        command += " {threshold}".format(**max_lsa)
    if max_lsa.get("warning_only"):
        command += " warning-only"
    if max_lsa.get("ignore_time"):
        command += " ignore-time {ignore_time}".format(**max_lsa)
    if max_lsa.get("ignore_count"):
        command += " ignore-count {ignore_count}".format(**max_lsa)
    if max_lsa.get("reset_time"):
        command += " reset-time {reset_time}".format(**max_lsa)
    return command


def _tmplt_max_metric(proc):
    max_metric = proc["max_metric"]
    command = "max-metric router-lsa"

    if max_metric.get("router_lsa", {}).get("set") is False:
        command = "no {0}".format(command)
    else:
        external_lsa = max_metric.get("router_lsa", {}).get("external_lsa", {})
        stub_prefix_lsa = max_metric.get("router_lsa", {}).get(
            "stub_prefix_lsa", {}
        )
        on_startup = max_metric.get("router_lsa", {}).get("on_startup", {})
        inter_area_prefix_lsa = max_metric.get("router_lsa", {}).get(
            "inter_area_prefix_lsa", {}
        )
        if external_lsa:
            command += " external-lsa"
            if external_lsa.get("max_metric_value"):
                command += " {max_metric_value}".format(**external_lsa)
        if stub_prefix_lsa:
            command += " stub-prefix-lsa"
        if on_startup:
            command += " on-startup"
            if on_startup.get("wait_period"):
                command += " {wait_period}".format(**on_startup)
            if on_startup.get("wait_for_bgp_asn"):
                command += " wait-for bgp {wait_for_bgp_asn}".format(
                    **on_startup
                )
        if inter_area_prefix_lsa:
            command += " inter-area-prefix-lsa"
            if inter_area_prefix_lsa.get("max_metric_value"):
                command += " {max_metric_value}".format(
                    **inter_area_prefix_lsa
                )

    return command


def _tmplt_area_ranges(arange):
    command = "area {area_id} range {prefix}".format(**arange)
    if arange.get("not_advertise") is True:
        command += " not-advertise"
    if "cost" in arange:
        command += " cost {cost}".format(**arange)
    return command


def _tmplt_default_information(proc):
    default_information = proc["default_information"]["originate"]
    command = "default-information originate"

    if default_information.get("set") is False:
        command = "no {0}".format(command)
    else:
        if default_information.get("always"):
            command += " always"
        if default_information.get("route_map"):
            command += " route-map {route_map}".format(**default_information)

    return command


def _tmplt_redistribute(redis):
    command = "redistribute {protocol}".format(**redis)
    if redis.get("id"):
        command += " {id}".format(**redis)
    command += " route-map {route_map}".format(**redis)
    return command


def _tmplt_summary_address(proc):
    command = "summary-address {prefix}".format(**proc)
    if proc.get("tag"):
        command += " tag {tag}".format(**proc)
    elif proc.get("not_advertise"):
        command += " not-advertise"
    return command


def _tmplt_table_map(proc):
    table_map = proc["table_map"]
    command = "table-map {name}".format(**table_map)
    if table_map.get("filter"):
        command += " filter"

    return command


class Ospfv3Template(NetworkTemplate):
    def __init__(self, lines=None):
        super(Ospfv3Template, self).__init__(lines=lines, tmplt=self)

    # fmt: off
    PARSERS = [
        {
            "name": "vrf",
            "getval": re.compile(
                r"""
                \s+vrf
                \s(?P<vrf>\S+)$""",
                re.VERBOSE,
            ),
            "setval": "vrf {{ vrf }}",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "vrf": "{{ vrf }}"
                    }
                }
            },
            "shared": True,
        },
        {
            "name": "process_id",
            "getval": re.compile(
                r"""
                ospfv3
                \s(?P<process_id>\S+)""",
                re.VERBOSE,
            ),
            "setval": "router ospfv3 {{ process_id }}",
            "result": {
                "process_id": "{{ process_id }}",
            },
            "shared": True,
        },
        {
            "name": "router_id",
            "getval": re.compile(
                r"""
                \s+router-id
                \s(?P<router_id>\S+)$""",
                re.VERBOSE,
            ),
            "setval": "router-id {{ router_id }}",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "router_id": "{{ router_id }}"
                    }
                }
            },
        },
        {
            "name": "address_family",
            "getval": re.compile(
                r"""
                \s+address-family
                \s(?P<afi>\S+)
                \s(?P<safi>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": "address-family {{ afi }} {{ safi }}",
            "result": {
                "address_family": {
                    "afi": "{{ afi }}",
                    "safi": "{{ safi }}",
                }
            },
            'shared': True,
        },
        {
            "name": "area.default_cost",
            "getval": re.compile(
                r"""
                \s+area\s(?P<area_id>\S+)\s
                default-cost\s(?P<default_cost>\d+)
                $""",
                re.VERBOSE,
            ),
            "setval": "area {{ area_id }} default-cost {{ default_cost }}",
            "compval": "default_cost",
            "result": {
                'address_family': {
                    "areas": {
                        "{{ area_id }}": {
                            "area_id": "{{ area_id }}",
                            "default_cost": "{{ default_cost|int }}",
                        }
                    }
                }
            },
        },
        {
            "name": "area.filter_list",
            "getval": re.compile(
                r"""
                \s+area\s(?P<area_id>\S+)
                \sfilter-list
                \sroute-map\s(?P<rmap>\S+)
                \s(?P<dir>\S+)$""",
                re.VERBOSE,
            ),
            "setval": "area {{ area_id }} filter-list route-map {{ route_map }} {{ direction }}",
            "result": {
                'address_family': {
                    "areas": {
                        "{{ area_id }}": {
                            "area_id": "{{ area_id }}",
                            "filter_list": [
                                {
                                    "route_map": "{{ rmap }}",
                                    "direction": "{{ dir }}",
                                },
                            ],
                        }
                    }
                }
            },
        },
        {
            "name": "area.ranges",
            "getval": re.compile(
                r"""
                \s+area\s(?P<area_id>\S+)
                \srange\s(?P<prefix>\S+)
                \s*(cost)*\s*(?P<cost>\d+)*
                \s*(?P<not_adver>not-advertise)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_area_ranges,
            "remval": "area {{ area_id }} range {{ prefix }}",
            "result": {
                "address_family": {
                    "areas": {
                        "{{ area_id }}": {
                            "area_id": "{{ area_id }}",
                            "ranges": [
                                {
                                    "prefix": "{{ prefix }}",
                                    "cost": "{{ cost }}",
                                    "not_advertise": "{{ not not not_adver }}",
                                },
                            ],
                        }
                    }
                }
            },
        },
        {
            "name": "default_information.originate",
            "getval": re.compile(
                r"""
                \s+default-information
                \s(?P<originate>originate)
                \s*(?P<always>always)*
                \s*(route-map)*
                \s*(?P<route_map>\S+)*$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_default_information,
            "result": {
                "address_family": {
                    "default_information": {
                        "originate": {
                            "set": "{{ True if originate is defined and always is undefined and route_map is undefined else None }}",
                            "always": "{{ not not always }}",
                            "route_map": "{{ route_map }}",
                        }
                    }
                }
            },
        },
        {
            "name": "distance",
            "getval": re.compile(
                r"""
                \s+distance
                \s(?P<distance>\d+)$""",
                re.VERBOSE,
            ),
            "setval": "distance {{ distance }}",
            "result": {
                "address_family": {
                    "distance": "{{ distance }}"
                }
            },
        },
        {
            "name": "maximum_paths",
            "getval": re.compile(
                r"""
                \s+maximum-paths
                \s(?P<maximum_paths>\d+)$""",
                re.VERBOSE,
            ),
            "setval": ("maximum-paths {{ maximum_paths }}"),
            "result": {
                "address_family": {
                    "maximum_paths": "{{ maximum_paths }}"
                }
            },
        },
        {
            "name": "redistribute",
            "getval": re.compile(
                r"""
                \s+redistribute
                \s(?P<protocol>\S+)
                \s*(?P<id>\S+)*
                \sroute-map\s(?P<rmap>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_redistribute,
            "result": {
                "address_family": {
                    "redistribute": [
                        {
                            "protocol": "{{ protocol }}",
                            "id": "{{ id }}",
                            "route_map": "{{ rmap }}",
                        },
                    ]
                }
            },
        },
        {
            "name": "summary_address",
            "getval": re.compile(
                r"""
                \s+summary-address
                \s(?P<prefix>\S+)
                \s*(?P<not_adver>not-advertise)*
                \s*(tag)*\s*(?P<tag>\d+)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_summary_address,
            "result": {
                "address_family": {
                    "summary_address": [
                        {
                            "prefix": "{{ prefix }}",
                            "not_advertise": "{{ not not not_adver }}",
                            "tag": "{{ tag }}",
                        },
                    ]
                }
            },
        },
        {
            "name": "table_map",
            "getval": re.compile(
                r"""
                \s+table-map
                \s(?P<rmap>\S+)
                \s*(?P<filter>filter)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_table_map,
            "result": {
                "address_family": {
                    "table_map": {
                        "name": "{{ rmap }}",
                        "filter": "{{ not not filter }}",
                    }
                }
            },
        },
        {
            "name": "timers.throttle.spf",
            "getval": re.compile(
                r"""
                \s+timers\sthrottle\sspf
                \s(?P<initial>\d+)
                \s(?P<min>\d+)
                \s(?P<max>\d+)
                $""",
                re.VERBOSE,
            ),
            "setval": "timers throttle spf {{ timers.throttle.spf.initial_spf_delay }}"
            " {{ timers.throttle.spf.min_hold_time }}"
            " {{ timers.throttle.spf.max_wait_time }}",
            "result": {
                "address_family": {
                    "timers": {
                        "throttle": {
                            "spf": {
                                "initial_spf_delay": "{{ initial }}",
                                "min_hold_time": "{{ min }}",
                                "max_wait_time": "{{ max }}",
                            }
                        }
                    }
                }
            },
        },
        {
            "name": "area.nssa",
            "getval": re.compile(
                r"""
                \s+area\s(?P<area_id>\S+)
                \s(?P<nssa>nssa)
                \s*(?P<no_sum>no-summary)*
                \s*(?P<no_redis>no-redistribution)*
                \s*(?P<def_info>default-information-originate)*
                \s*(route-map)*\s*(?P<rmap>\S+)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_area_nssa,
            "remval": "area {{ area_id }} nssa",
            "compval": "nssa",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "nssa": {
                                    "set": "{{ True if nssa is defined and no_sum is undefined and no_redis is undefined and \
                                        def_info is undefined and rmap is undefined }}",
                                    "no_summary": "{{ not not no_sum }}",
                                    "no_redistribution": "{{ not not no_redis }}",
                                    "default_information_originate": "{{ not not def_info }}",
                                    "route_map": "{{ rmap }}",
                                },
                            }
                        }
                    }
                }
            },
        },
        {
            "name": "area.nssa.translate",
            "getval": re.compile(
                r"""
                \s+area\s(?P<area_id>\S+)\snssa
                \stranslate
                \stype7
                \s(?P<choice>always|never)
                \s*(?P<supress_fa>supress-fa)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_area_nssa_translate,
            "compval": "nssa.translate",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "nssa": {
                                    "translate": {
                                        "type7": {
                                            "always": '{{ True if choice == "always" else None }}',
                                            "never": '{{ True if choice == "never" else None }}',
                                            "supress_fa": "{{ not not supress_fa }}",
                                        }
                                    }
                                },
                            }
                        }
                    }
                }
            },
        },
        {
            "name": "area.stub",
            "getval": re.compile(
                r"""
                \s+area\s(?P<area_id>\S+)
                \s(?P<stub>stub)
                \s*(?P<no_summary>no-summary)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_area_stub,
            "remval": "area {{ area_id }} stub",
            "compval": "stub",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "stub": {
                                    "set": "{{ True if stub is defined and no_summary is undefined else None }}",
                                    "no_summary": "{{ not not no_summary }}",
                                },
                            }
                        }
                    }
                }
            },
        },
        {
            "name": "area.virtual_link",
            "getval": re.compile(
                r"""
                \s+area\s(?P<area_id>\S+)
                \svirtual-link
                \s(?P<virtual_link>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": "area {{ area_id }} virtual-link {{ virtual_link }}",
            "compval": "virtual_link",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "virtual_link": "{{ virtual_link }}",
                            },
                        }
                    }
                }
            },
        },
        {
            "name": "auto_cost",
            "getval": re.compile(
                r"""
                \s+auto-cost\sreference-bandwidth\s
                (?P<acrb>\d+)\s(?P<unit>\S+)$""",
                re.VERBOSE,
            ),
            "setval": (
                "auto-cost reference-bandwidth"
                " {{ auto_cost.reference_bandwidth }}"
                " {{ auto_cost.unit }}"
            ),
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "auto_cost": {
                            "reference_bandwidth": "{{ acrb }}",
                            "unit": "{{ unit }}",
                        }
                    }
                }
            },
        },
        {
            "name": "flush_routes",
            "getval": re.compile(
                r"""
                \s+(?P<flush_routes>flush-routes)$""",
                re.VERBOSE,
            ),
            "setval": "flush-routes",
            "result": {
                "flush_routes": "{{ not not flush_routes }}"
            },
        },
        {
            "name": "graceful_restart.set",
            "getval": re.compile(
                r"""
                \s+(?P<graceful_restart>no\sgraceful-restart)
                $""",
                re.VERBOSE,
            ),
            "setval": "graceful-restart",
            "remval": "no graceful-restart",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "graceful_restart": {
                            "set": "{{ not graceful_restart }}"
                        }
                    }
                }
            },
        },
        {
            "name": "graceful_restart.helper_disable",
            "getval": re.compile(
                r"""
                \s+graceful-restart
                \s+(?P<helper_disable>helper-disable)
                $""",
                re.VERBOSE,
            ),
            "setval": "graceful-restart helper-disable",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "graceful_restart": {
                            "helper_disable": "{{ not not helper_disable }}",
                        }
                    }
                }
            },
        },
        {
            "name": "graceful_restart.grace_period",
            "getval": re.compile(
                r"""
                \s+graceful-restart
                \s+grace-period
                \s+(?P<grace_period>\d+)
                $""",
                re.VERBOSE,
            ),
            "setval": "graceful-restart grace-period {{ graceful_restart.grace_period }}",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "graceful_restart": {
                            "grace_period": "{{ grace_period }}",
                        }
                    }
                }
            },
        },
        {
            "name": "graceful_restart.planned_only",
            "getval": re.compile(
                r"""
                \s+no
                \s+graceful-restart
                \s+(?P<planned_only>planned-only)
                $""",
                re.VERBOSE,
            ),
            "setval": "graceful-restart planned-only",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "graceful_restart": {
                            "planned_only": "{{ not planned_only }}",
                        }
                    }
                }
            },
        },
        {
            "name": "isolate",
            "getval": re.compile(
                r"""
                \s+(?P<isolate>isolate)$""",
                re.VERBOSE,
            ),
            "setval": "isolate",
            "result": {"isolate": "{{ not not isolate }}"},
        },
        {
            "name": "log_adjacency_changes",
            "getval": re.compile(
                r"""
                \s+(?P<log>log-adjacency-changes)
                \s*(?P<detail>detail)*$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_log_adjacency_changes,
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "log_adjacency_changes": {
                            "log": "{{ True if log is defined and detail is undefined else None }}",
                            "detail": "{{ True if detail is defined else None }}",
                        }
                    }
                }
            },
        },
        {

            "name": "max_lsa",
            "getval": re.compile(
                r"""
                \s+max-lsa
                \s(?P<max_gen_lsa>\d+)
                \s*(?P<threshold>\d*)
                \s*(?P<warning_only>warning-only)*
                \s*(ignore-time)*\s*(?P<ig_time>\d*)
                \s*(ignore-count)*\s*(?P<ig_count>\d*)
                \s*(reset-time)*\s*(?P<rst_time>\d*)
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_max_lsa,
            "remval": "max-lsa {{ max_lsa.max_non_self_generated_lsa }}",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "max_lsa": {
                            "max_non_self_generated_lsa": "{{ max_gen_lsa }}",
                            "threshold": "{{ threshold }}",
                            "ignore_time": "{{ ig_time }}",
                            "ignore_count": "{{ ig_count }}",
                            "reset_time": "{{ rst_time }}",
                            "warning_only": "{{ not not warning_only }}",
                        }
                    }
                }
            },
        },
        {
            "name": "max_metric",
            "getval": re.compile(
                r"""
                \s+max-metric
                \s+(?P<router_lsa>router-lsa)
                \s*(?P<external_lsa>external-lsa)*
                \s*(?P<max_metric_value>\d+)*
                \s*(?P<stub_prefix_lsa>stub-prefix-lsa)*
                \s*(?P<on_startup>on-startup)*
                \s*(?P<wait_period>\d+)*
                \s*(wait-for\sbgp)*
                \s*(?P<bgp_asn>\d+)*
                \s*(?P<inter_area_prefix_lsa>inter-area-prefix-lsa)*
                \s*(?P<max_metric_summary_lsa>\d+)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_max_metric,
            "remval": "max-metric router-lsa",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "max_metric": {
                            "router_lsa": {
                                "set": "{{ True if router_lsa is defined and (external_lsa is undefined) and (inter_area_prefix_lsa is undefined) and \
                                    (stub_prefix_lsa is undefined) and (on_startup is undefined) else None }}",
                                "external_lsa": {
                                    "set": "{{ True if external_lsa is defined and max_metric_value is undefined else None }}",
                                    "max_metric_value": "{{ max_metric_value }}",
                                },
                                "stub_prefix_lsa": "{{ not not stub_prefix_lsa }}",
                                "on_startup": {
                                    "set": "{{ True if on_startup is defined and (wait_period and bgp_asn) is undefined else None }}",
                                    "wait_period": "{{ wait_period }}",
                                    "wait_for_bgp_asn": "{{ bgp_asn }}",
                                },
                                "inter_area_prefix_lsa": {
                                    "set": "{{ True if inter_area_prefix_lsa is defined and max_metric_summary_lsa is undefined else None }}",
                                    "max_metric_value": "{{ max_metric_summary_lsa }}",
                                },
                            }
                        }
                    }
                }
            },
        },
        {
            "name": "name_lookup",
            "getval": re.compile(
                r"""
                \s+(?P<name_lookup>name-lookup)
                $""",
                re.VERBOSE,
            ),
            "setval": ("name-lookup"),
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "name_lookup": "{{ not not name_lookup }}"
                    }
                }
            },
        },
        {
            "name": "passive_interface.default",
            "getval": re.compile(
                r"""
                \s+passive-interface
                \s+(?P<default>default)
                $""",
                re.VERBOSE,
            ),
            "setval": ("passive-interface default"),
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "passive_interface": {"default": "{{ not not default }}"}
                    }
                }
            },
        },
        {
            "name": "shutdown",
            "getval": re.compile(
                r"""
                \s+(?P<shutdown>shutdown)$""",
                re.VERBOSE,
            ),
            "setval": ("shutdown"),
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "shutdown": "{{ not not shutdown }}"
                    }
                }
            },
        },
        {
            "name": "timers.lsa_arrival",
            "getval": re.compile(
                r"""
                \s+timers
                \slsa-arrival
                \s(?P<lsa_arrival_val>\d+)
                $""",
                re.VERBOSE,
            ),
            "setval": ("timers lsa-arrival {{ timers.lsa_arrival }}"),
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "timers": {
                            "lsa_arrival": "{{ lsa_arrival_val }}"
                        }
                    }
                }
            },
        },
        {
            "name": "timers.lsa_group_pacing",
            "getval": re.compile(
                r"""
                \s+timers
                \slsa-group-pacing
                \s(?P<lsa_group_pacing>\d+)
                $""",
                re.VERBOSE,
            ),
            "setval": "timers lsa-group-pacing {{ timers.lsa_group_pacing }}",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "timers": {
                            "lsa_group_pacing": "{{ lsa_group_pacing }}"
                        }
                    }
                }
            },
        },
        {
            "name": "timers.throttle.lsa",
            "getval": re.compile(
                r"""
                \s+timers\sthrottle\slsa
                \s(?P<start>\d+)
                \s(?P<hold>\d+)
                \s(?P<max>\d+)
                $""",
                re.VERBOSE,
            ),
            "setval": "timers throttle lsa {{ timers.throttle.lsa.start_interval }}"
            " {{ timers.throttle.lsa.hold_interval }}"
            " {{ timers.throttle.lsa.max_interval }}",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "timers": {
                            "throttle": {
                                "lsa": {
                                    "start_interval": "{{ start }}",
                                    "hold_interval": "{{ hold }}",
                                    "max_interval": "{{ max }}",
                                }
                            }
                        }
                    }
                }
            },
        },
    ]
    # fmt: on



# (c) 2017 Red Hat Inc.
#
# This file is part of Ansible
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

# Make coding more python3-ish
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from ansible_collections.amazon.aws.tests.unit.utils.amazon_placebo_fixtures import placeboify, maybe_sleep
from ansible_collections.community.aws.plugins.modules import aws_direct_connect_virtual_interface


class FakeModule(object):
    def __init__(self, **kwargs):
        self.params = kwargs

    def fail_json(self, *args, **kwargs):
        self.exit_args = args
        self.exit_kwargs = kwargs
        raise Exception("FAIL")

    def exit_json(self, *args, **kwargs):
        self.exit_args = args
        self.exit_kwargs = kwargs


def test_find_unique_vi_by_connection_id(placeboify, maybe_sleep):
    client = placeboify.client("directconnect")
    vi_id = aws_direct_connect_virtual_interface.find_unique_vi(client, "dxcon-aaaaaaaa", None, None)
    assert vi_id == "dxvif-aaaaaaaa"


def test_find_unique_vi_by_vi_id(placeboify, maybe_sleep):
    client = placeboify.client("directconnect")
    vi_id = aws_direct_connect_virtual_interface.find_unique_vi(client,
                                                                None,
                                                                "dxvif-aaaaaaaaa",
                                                                None)
    assert vi_id == "dxvif-aaaaaaaa"


def test_find_unique_vi_by_name(placeboify, maybe_sleep):
    client = placeboify.client("directconnect")
    vi_id = aws_direct_connect_virtual_interface.find_unique_vi(client, None, None, "aaaaaaaa")
    assert vi_id == "dxvif-aaaaaaaa"


def test_find_unique_vi_returns_multiple(placeboify, maybe_sleep):
    client = placeboify.client("directconnect")
    module = FakeModule(state="present",
                        id_to_associate="dxcon-aaaaaaaa",
                        public=False,
                        name=None)
    try:
        aws_direct_connect_virtual_interface.ensure_state(
            client,
            module
        )
    except Exception:
        assert "Multiple virtual interfaces were found" in module.exit_kwargs["msg"]


def test_find_unique_vi_returns_missing_for_vi_id(placeboify, maybe_sleep):
    client = placeboify.client("directconnect")
    module = FakeModule(state="present",
                        id_to_associate=None,
                        public=False,
                        name=None,
                        virtual_interface_id="dxvif-aaaaaaaa")
    try:
        aws_direct_connect_virtual_interface.ensure_state(
            client,
            module
        )
    except Exception:
        assert "The virtual interface dxvif-aaaaaaaa does not exist" in module.exit_kwargs["msg"]


def test_construct_public_vi():
    module = FakeModule(state="present",
                        id_to_associate=None,
                        public=True,
                        name="aaaaaaaa",
                        vlan=1,
                        bgp_asn=123,
                        authentication_key="aaaa",
                        customer_address="169.254.0.1/30",
                        amazon_address="169.254.0.2/30",
                        address_type="ipv4",
                        cidr=["10.88.0.0/30"],
                        virtual_gateway_id="xxxx",
                        direct_connect_gateway_id="yyyy")
    vi = aws_direct_connect_virtual_interface.assemble_params_for_creating_vi(module.params)
    assert vi == {
        "virtualInterfaceName": "aaaaaaaa",
        "vlan": 1,
        "asn": 123,
        "authKey": "aaaa",
        "amazonAddress": "169.254.0.2/30",
        "customerAddress": "169.254.0.1/30",
        "addressFamily": "ipv4",
        "routeFilterPrefixes": [{"cidr": "10.88.0.0/30"}]
    }


def test_construct_private_vi_with_virtual_gateway_id():
    module = FakeModule(state="present",
                        id_to_associate=None,
                        public=False,
                        name="aaaaaaaa",
                        vlan=1,
                        bgp_asn=123,
                        authentication_key="aaaa",
                        customer_address="169.254.0.1/30",
                        amazon_address="169.254.0.2/30",
                        address_type="ipv4",
                        cidr=["10.88.0.0/30"],
                        virtual_gateway_id="xxxx",
                        direct_connect_gateway_id="yyyy")
    vi = aws_direct_connect_virtual_interface.assemble_params_for_creating_vi(module.params)
    assert vi == {
        "virtualInterfaceName": "aaaaaaaa",
        "vlan": 1,
        "asn": 123,
        "authKey": "aaaa",
        "amazonAddress": "169.254.0.2/30",
        "customerAddress": "169.254.0.1/30",
        "addressFamily": "ipv4",
        "virtualGatewayId": "xxxx"
    }


def test_construct_private_vi_with_direct_connect_gateway_id():
    module = FakeModule(state="present",
                        id_to_associate=None,
                        public=False,
                        name="aaaaaaaa",
                        vlan=1,
                        bgp_asn=123,
                        authentication_key="aaaa",
                        customer_address="169.254.0.1/30",
                        amazon_address="169.254.0.2/30",
                        address_type="ipv4",
                        cidr=["10.88.0.0/30"],
                        virtual_gateway_id=None,
                        direct_connect_gateway_id="yyyy")
    vi = aws_direct_connect_virtual_interface.assemble_params_for_creating_vi(module.params)
    print(vi)
    assert vi == {
        "virtualInterfaceName": "aaaaaaaa",
        "vlan": 1,
        "asn": 123,
        "authKey": "aaaa",
        "amazonAddress": "169.254.0.2/30",
        "customerAddress": "169.254.0.1/30",
        "addressFamily": "ipv4",
        "directConnectGatewayId": "yyyy"
    }


def test_create_public_vi(placeboify, maybe_sleep):
    client = placeboify.client("directconnect")
    module = FakeModule(state="present",
                        id_to_associate='dxcon-aaaaaaaa',
                        virtual_interface_id=None,
                        public=True,
                        name="aaaaaaaa",
                        vlan=1,
                        bgp_asn=123,
                        authentication_key="aaaa",
                        customer_address="169.254.0.1/30",
                        amazon_address="169.254.0.2/30",
                        address_type="ipv4",
                        cidr=["10.88.0.0/30"],
                        virtual_gateway_id="xxxx",
                        direct_connect_gateway_id="yyyy")
    changed, latest_state = aws_direct_connect_virtual_interface.ensure_state(client, module)
    assert changed is True
    assert latest_state is not None


def test_create_private_vi(placeboify, maybe_sleep):
    client = placeboify.client("directconnect")
    module = FakeModule(state="present",
                        id_to_associate='dxcon-aaaaaaaa',
                        virtual_interface_id=None,
                        public=False,
                        name="aaaaaaaa",
                        vlan=1,
                        bgp_asn=123,
                        authentication_key="aaaa",
                        customer_address="169.254.0.1/30",
                        amazon_address="169.254.0.2/30",
                        address_type="ipv4",
                        cidr=["10.88.0.0/30"],
                        virtual_gateway_id="xxxx",
                        direct_connect_gateway_id="yyyy")
    changed, latest_state = aws_direct_connect_virtual_interface.ensure_state(client, module)
    assert changed is True
    assert latest_state is not None


def test_delete_vi(placeboify, maybe_sleep):
    client = placeboify.client("directconnect")
    module = FakeModule(state="absent",
                        id_to_associate='dxcon-aaaaaaaa',
                        virtual_interface_id='dxvif-aaaaaaaa',
                        public=False,
                        name="aaaaaaaa",
                        vlan=1,
                        bgp_asn=123,
                        authentication_key="aaaa",
                        customer_address="169.254.0.1/30",
                        amazon_address="169.254.0.2/30",
                        address_type="ipv4",
                        cidr=["10.88.0.0/30"],
                        virtual_gateway_id=None,
                        direct_connect_gateway_id="yyyy")
    changed, latest_state = aws_direct_connect_virtual_interface.ensure_state(client, module)
    assert changed is True
    assert latest_state == {}
